from dal import autocomplete
from django import forms
from upload_validator import FileTypeValidator

from audiothek.models import Audiofile, RadioReport, RadioReportRole
from audiothek.processing import Recording
from audiothek.tasks import process_recording
from audiothek.overrides import taggit_select2_format_value,\
    taggit_select2_options
from core.models import Image
from taggit.utils import edit_string_for_tags
from flatpickr import DateTimePickerInput

# monkey patch dal_select2 autocomplete of tags
autocomplete.TagSelect2.format_value = taggit_select2_format_value
autocomplete.TagSelect2.options = taggit_select2_options


class AudiofileFromRecordingForm(forms.ModelForm):
    class Meta:
        model = Audiofile
        fields = ('title', 'description')

    start_time = forms.DateTimeField(label='Beginn der Aufzeichnung')
    end_time = forms.DateTimeField(label='Ende der Aufzeichnung')

    def save(self, commit=True):
        audiofile = super().save(commit)
        recording = Recording(
                audiofile,
                self.cleaned_data['start_time'],
                self.cleaned_data['end_time'])
        process_recording(recording)
        return audiofile


class RadioReportForm(forms.ModelForm):
    image_file = forms.ImageField(label='Bild', required=False)

    class Meta:
        model = RadioReport
        help_texts = {
            'title': "Wähle einen prägnaten Titel zum Inhalt des Beitrages. \
            Empfehlung: max. 8 Worte",
            'categories': 'Mehrfachauswahl und Abwahl mit STRG',
            'description': 'Enthält dein Beitrag Drittmaterial, \
            schreibe Quellen/Lizenzen zuletzt unter deine Beschreibung',
            'publication_time': 'bei leerem Feld bleibt der Beitrag privat',
            'publicationlimit': 'Beitrag ist nur temporär veröffentlichbar',
            'externalcreators': 'Optional. Hier nur externe (nicht registrierte) Nutzer*innen. \
            Interne Urheber werden über Rollen zugewiesen.',
            'tags': 'Erstelle treffende Schlagwörter, dass wenn Nutzer*innen danach suchen, \
            auch diesen Beitrag als Suchergebnis erwarten würden.',

        }
        fields = (
                'title', 'description', 'production_date', 'tags', 'publication_time',
                'license', 'type', 'categories', 'publicationlimit', 'externalcreators'
        )
        widgets = {
            'categories': forms.SelectMultiple(attrs={'size': 18, 'label': 'Themenfelder'}),
            'description': forms.Textarea(attrs={'rows': 5}),
            'tags': autocomplete.TaggitSelect2(
                'tag-autocomplete', attrs={'data-theme': 'bootstrap4'},
            ),
            'publication_time': DateTimePickerInput(),
        }

    def __init__(self, *args, **kwargs):
        super(RadioReportForm, self).__init__(*args, **kwargs)
        if self.instance.pk:
            self.fields['tags'].initial = edit_string_for_tags(self.instance.tags.all())
        self.fields['categories'].label = 'Themenfelder'

    def save(self, commit=True):
        if self.cleaned_data['image_file']:
            self.instance.cover_image = Image.objects.create(
                    image_file=self.cleaned_data['image_file'])
        return super().save(commit)


class RadioReportQuickForm(forms.ModelForm):
    audiofile = forms.FileField(label='upload audio')

    class Meta:
        model = RadioReport
        fields = ('title', 'description', 'audiofile')
        widgets = {
            'description': forms.Textarea(attrs={'rows': 3}),
            }


class AudiofileForm(forms.ModelForm):

    audiofile = forms.FileField(
        label='Audiodatei', help_text="Akzeptiertes Format: MP3",
        required=False,
        validators=[
            FileTypeValidator(
                allowed_types=['audio/mpeg', 'audio/mp3'],
                allowed_extensions=['.mp3'],
            )
        ]
    )

    class Meta:
        model = Audiofile
        fields = ['title', 'description', 'audiofile']
        widgets = {
            'description': forms.Textarea(attrs={'rows': 3})
        }


class RadioReportRoleForm(forms.ModelForm):

    class Meta:
        model = RadioReportRole
        fields = ['name', 'user']
        labels = {
            'user': 'Nutzer*in',
            'name': 'Tätigkeit',
        }
        widgets = {
            'user': autocomplete.ModelSelect2(url='user-autocomplete')
        }
