from pathlib import Path
import sys

from django.conf import settings
from django.core.management.base import BaseCommand
import fasteners

from audiothek.models import Audiofile
from audiothek.processing import resolve_formats, get_audio_metadata, transcode


def _progress(iterable, total, output, force_disable):
    class RemoveEndings:
        def write(self, *args, **kwargs):
            output.write(*args, ending='', **kwargs)
    try:
        from tqdm import tqdm
        return tqdm(iterable, file=RemoveEndings(), total=total, disable=force_disable)
    except ImportError:
        return iterable


def transcode_files(qs, stdout, force_disable):
    formats = resolve_formats(settings.LOHROTHEK_TRANSCODE_FORMATS)
    for audio_file in _progress(qs, qs.count(), stdout, force_disable):
        encoded_files = transcode(audio_file.audiofile.path, formats)
        for encoded_file in encoded_files:
            metadata = get_audio_metadata(encoded_file.path)
            path = Path(encoded_file.path)
            if path.is_absolute():
                path = path.relative_to(audio_file.audiofile.storage.location)
            Audiofile.objects.create(
                creator=audio_file.creator,
                title=audio_file.title,
                description=audio_file.description,
                audiofile=str(path),
                transcoded_from=audio_file,
                **metadata.as_dict())


class Command(BaseCommand):
    help = 'Transcodes audiofiles to defined formats'

    def add_arguments(self, parser):
        parser.add_argument('audio_file_id', nargs='*', type=int,
                            help='One or more AudioFile ids. '
                                 'Defaults to all audio files in the database.')
        parser.add_argument('--show-missing', action='store_true',
                            help='List the paths of files that have yet to be transcoded.')
        parser.add_argument('--lock', default=None, type=str,
                            help='Make sure that only a single transcoding process runs '
                                 'at a time. You must provide a path that will be used '
                                 'as a lock file.')
        parser.add_argument('--lock-exit-code', default=6, type=int,
                            help='Exit code if a lock cannot be acquired.')
        parser.add_argument('--no-progress', action='store_true', default=not self.stdout.isatty(),
                            help='Disable any progress indication during transcoding.')

    def handle(self, *args, **options):
        file_ids = options.get('audio_file_id', [])
        qs = Audiofile.get_untranscoded_audiofiles()
        if file_ids:
            qs = qs.filter(id__in=file_ids)

        if options['show_missing']:
            for audio_file in qs:
                self.stdout.write(audio_file.audiofile.path)
            return

        def _transcode():
            transcode_files(qs, self.stdout, options['no_progress'])

        if options['lock']:
            transcode_lock = fasteners.InterProcessLock(options['lock'])
            if transcode_lock.acquire(blocking=False):
                try:
                    _transcode()
                finally:
                    transcode_lock.release()
            else:
                sys.exit(options['lock_exit_code'])
        else:
            _transcode()
