from datetime import date, datetime, timedelta
from enum import Enum
import pytz

from django.conf import settings
from django.db import models
from django.db.models import Exists, OuterRef
from django.urls import reverse
from mptt.models import MPTTModel, TreeForeignKey, TreeManyToManyField
from taggit.managers import TaggableManager
from core.models import SoftDeleteModel, ContentTag
from django.dispatch.dispatcher import receiver
from django.db.models.signals import post_save
from lohrothek import settings as theksettings
from audiothek.util import rename_audiofile_filename, build_radioreport_changes, \
    build_audiofile_changes, move_previous_audiofile
from model_utils.tracker import FieldTracker
from program.models import Broadcast
from taggit.models import GenericTaggedItemBase
from mptt.managers import TreeManager
from django.utils import timezone


PUBLICATION_LIMIT_7D = '7d'
PUBLICATION_LIMITS = ((PUBLICATION_LIMIT_7D, 'GEMA ±7 Tage Ausstrahlung'),)
RADIOREPORT_AUDIOFILE_PATH_RENAME_TRACKFIELDS = ['production_date', 'title']
AUDIOFILE_PATH_RENAME_TRACKFIELDS = ['audiofile']


def django_enum(cls):
    # decorator needed to enable enums in django templates
    cls.do_not_call_in_templates = True
    return cls


class RadioReportCategoryCountManager(TreeManager):

    def get_queryset(self, *args, **kwargs):
        return RadioReportCategory.objects.add_related_count(
            super(RadioReportCategoryCountManager, self).get_queryset(),
            RadioReport, 'categories', 'related_count', cumulative=False
            )


class RadioReportCategory(MPTTModel):
    name = models.CharField('Name', max_length=255)
    description = models.TextField('Beschreibung', blank=True)
    parent = TreeForeignKey(
        'self', on_delete=models.CASCADE, null=True, blank=True, related_name='children')

    # init default model manager when using additional manager(s)
    objects = TreeManager()
    # init additional model manager
    objects_report_count = RadioReportCategoryCountManager()

    class MPTTMeta:
        order_insertion_by = ['name']

    def __str__(self):
        return self.name


class RadioReportRole(models.Model):

    @django_enum
    class Name(Enum):
        every = '- alles -'
        interviewer = 'Interview'
        editor = 'Redaktion'
        cutter = 'Schnitt'
        voice = 'Stimme'
        translator = 'Übersetzung'

        def __repr__(self):
            return self.name

        def __str__(self):
            return self.value

    name = models.CharField(max_length=31, choices=[(role.name, role.value) for role in Name])
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.PROTECT)
    report = models.ForeignKey('RadioReport', on_delete=models.CASCADE,
                               null=True, blank=True, related_name='roles')
    public = models.BooleanField('öffentlich sichtbar', blank=True, default=True)

    def get_name(self):
        return self.Name[self.name]


@django_enum
class RadioReportType(Enum):
    report = 'Beitrag'
    radio_play = 'Hörspiel'
    interview = 'Interview'
    comment = 'Kommentar'
    podcast = 'Podcast'
    broadcast_recording = 'Sendungsmitschnitt'
    studio_interview = 'Studiointerview'

    def __repr__(self):
        return self.name

    def __str__(self):
        return self.value


class RadioReportContentTag(GenericTaggedItemBase):

    tag = models.ForeignKey(
        ContentTag,
        on_delete=models.CASCADE,
        related_name="%(app_label)s_%(class)s_items",
    )


class RadioReport(SoftDeleteModel):

    @django_enum
    class PublicationStatus(Enum):
        private = "private"
        public = "public"
        public_temp = "public-temp"
        public_outoflimit = "public-outoflimit"
        scheduled = "scheduled"

    type = models.CharField(
        max_length=31,
        choices=[(type.name, type.value) for type in RadioReportType],
        default=RadioReportType.report,
        null=True
    )
    title = models.CharField('Titel', max_length=255)
    description = models.TextField('Beschreibung', null=True, blank=True)
    categories = TreeManyToManyField(RadioReportCategory, blank=True)
    intro = models.TextField('Anmoderation', blank=True)
    outro = models.TextField('Abmoderation', blank=True)
    production_date = models.DateField(
            'Produktionsdatum', default=date.today, null=True, blank=True,
            help_text='Datum der Fertigstellung des Beitrags')
    publication_time = models.DateTimeField(
            'Veröffentlichung (Datum/Uhrzeit)', null=True, blank=True)
    unpublication_time = models.DateTimeField(
            'Unveröffentlichung (Datum/Uhrzeit)', null=True, blank=True)
    publicationlimit = models.CharField(
        'Veröffentlichungsbeschränkung', blank=True, max_length=255, choices=PUBLICATION_LIMITS)
    tags = TaggableManager(blank=True, through=RadioReportContentTag)
    license = models.ForeignKey(
            'core.License', on_delete=models.PROTECT, verbose_name='Lizenz', null=True, blank=True)
    externalcreators = models.CharField('Externe Urheber(innen)', max_length=255, blank=True,
                                        null=True)
    cover_image = models.ForeignKey('core.Image', on_delete=models.SET_NULL,
                                    null=True, blank=True)
    creator = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.PROTECT, null=True,
                                related_name='report_creator')
    creation_time = models.DateTimeField('erstellt', default=timezone.now, blank=True, null=True)
    last_editor = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.PROTECT, null=True,
                                    related_name='report_last_editor')
    last_edit_time = models.DateTimeField('zuletzt geändert', blank=True, null=True)

    tracker = FieldTracker()

    def __str__(self):
        if self.title:
            return self.title
        return "(kein Titel)"

    def get_absolute_url(self):
        return reverse('show-report', args=[self.pk])

    @property
    def can_be_streamed(self):
        return bool(
            self.publication_time and self.publication_time < timezone.now()
            and (not self.unpublication_time or self.unpublication_time > timedelta.now())
            and self.audiofiles.exists()
        )

    @property
    def publicationstatus(self):
        if self.publication_time is None:
            return self.PublicationStatus.private

        now = pytz.timezone(settings.TIME_ZONE).localize(datetime.now())

        if len(self.publicationlimit) == 0:
            if self.publication_time <= now:
                return self.PublicationStatus.public
            return self.PublicationStatus.scheduled

        else:
            if self.publicationlimit == PUBLICATION_LIMIT_7D:
                if Broadcast.objects.filter(items__report=self)\
                   .filter(start_time__gte=now - timedelta(7))\
                   .filter(start_time__lte=now + timedelta(7)):
                    return self.PublicationStatus.public_temp
                else:
                    if Broadcast.objects.filter(items__report=self)\
                       .filter(start_time__gt=now + timedelta(7)):
                        return self.PublicationStatus.scheduled
                    else:
                        return self.PublicationStatus.public_outoflimit


class Audiofile(SoftDeleteModel):
    # TODO: we should use the new UserRole system here as well. see RadioReport
    position = models.PositiveSmallIntegerField(default=1)
    creator = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.PROTECT)
    report = models.ForeignKey('RadioReport', on_delete=models.CASCADE, related_name='audiofiles',
                               null=True)
    title = models.CharField('Titel', max_length=255, blank=True)
    description = models.TextField('Beschreibung', blank=True)
    audiofile = models.FileField('Audiodatei', upload_to=theksettings.AUDIO_SUBFOLDER,
                                 max_length=255)
    length = models.PositiveSmallIntegerField(default=0)
    bitrate = models.PositiveIntegerField(default=0)
    channels = models.PositiveSmallIntegerField(default=0)
    samplerate = models.PositiveIntegerField(default=0, null=True)
    transcoded_from = models.ForeignKey(
            'Audiofile', null=True, blank=True, on_delete=models.CASCADE,
            related_name='transcodings')
    container = models.CharField(max_length=20, blank=True, null=True)
    codec = models.CharField(max_length=20, blank=True)

    tracker = FieldTracker()

    class Meta:
        ordering = ['position']

    def __str__(self):
        return self.title or '(Ohne Titel, ID:{})'.format(self.pk)

    def __iter__(self):
        yield self
        yield from self.transcodings.all()

    def save(self, *args, **kwargs):
        if not self.pk:
            audiofilesAtReport = Audiofile.objects.filter(report=self.report)
            self.position = audiofilesAtReport.count() + 1
        super(Audiofile, self).save()

    def delete(self):
        existHigherPos = Audiofile.objects.filter(
            report=self.report).filter(position__gt=self.position)
        if existHigherPos:
            for af in existHigherPos:
                af.position -= 1
                af.save()
        super(Audiofile, self).delete()

    def get_absolute_url(self):
        return reverse('show-report', args=[self.report.pk])

    @classmethod
    def get_untranscoded_audiofiles(cls):
        has_transcoded = Exists(cls.objects.filter(transcoded_from=OuterRef('pk')))
        return cls.objects\
            .annotate(has_transcoded=has_transcoded)\
            .filter(transcoded_from=None, has_transcoded=False)


@receiver(post_save, sender=RadioReport)
@receiver(post_save, sender=Audiofile)
def rewrite_audiofile_path(sender, instance, created, **kwargs):

    updateable = []

    if type(instance) == Audiofile:

        # exit on audiofile soft delete
        if instance.deletion_time:
            return

        # new file uploaded
        if build_audiofile_changes(instance, AUDIOFILE_PATH_RENAME_TRACKFIELDS):
            prev_audiofile = instance.tracker.previous('audiofile')

            # one previous uploaded file, move old to dump folder
            if prev_audiofile:
                move_previous_audiofile(instance, prev_audiofile.name)

            # put new uploaded file for renaming
            updateable.append(instance)

    elif type(instance) == RadioReport:

        # exit on radio report creation or deletion
        if created or instance.deletion_time:
            return

        # only on report title or production date changed
        if build_radioreport_changes(instance, RADIOREPORT_AUDIOFILE_PATH_RENAME_TRACKFIELDS):
            for af in instance.audiofiles.all():
                updateable.append(af)
    else:
        return

    if not updateable:
        return

    # temporarily disconnect self from Audiofile sender to avoid recursion with save() at model

    post_save.disconnect(rewrite_audiofile_path, sender=Audiofile)
    for u in updateable:
        rename_audiofile_filename(u)
    post_save.connect(rewrite_audiofile_path, sender=Audiofile)
