from collections import namedtuple
import enum
from hashlib import sha1
import logging
import os
from pathlib import Path
import subprocess
from tempfile import TemporaryDirectory, NamedTemporaryFile
from typing import Iterable, Iterator, List, Sequence, Union

from django.conf import settings
from django.core.files import File
from django.utils.dateparse import parse_datetime
import mutagen
import requests

logger = logging.getLogger(__name__)

FilePath = Union[Path, str]
EncoderInfo = namedtuple('EncoderInfo', ('format_name', 'container', 'codec', 'file_suffix'))
EncodedFile = namedtuple('EncodedFile', ('path', 'encoder_info'))


class AudioMetadata(namedtuple('_', ('container', 'codec', 'bitrate',
                                     'channels', 'length', 'samplerate'))):
    def as_dict(self):
        return {key: getattr(self, key) for key in self._fields}

    def assign_to(self, audio_file):
        for key, value in self.as_dict().items():
            setattr(audio_file, key, value)


class Recording:
    def __init__(self, model, start_time, end_time):
        self.model = model
        self.start_time = start_time
        self.end_time = end_time

    def process(self):
        self.snippets = self._get_snippets()
        self.audiofile = NamedTemporaryFile(suffix='.mp3')
        with TemporaryDirectory() as dirname:
            filepath = os.path.join(dirname, 'recording.mp3')
            self._copy_snippets(dirname)
            self._concat_snippets(filepath)
            self._trim_audiofile(filepath)
        self.model.audiofile.save('recording.mp3', File(self.audiofile))

    def _get_snippets(self):
        params = {'end_time__gt': self.start_time, 'start_time__lt': self.end_time}
        r = requests.get(settings.LOHROTHEK_RECORDING_API_URL, params=params)
        return r.json()

    def _copy_snippets(self, dirname):
        for snippet in self.snippets:
            r = requests.get(snippet['file_ref'])
            filename = '{}.mp3'.format(sha1(r.content).hexdigest()[:16])
            local_path = os.path.join(dirname, filename)
            with open(local_path, 'wb') as f:
                f.write(r.content)
            snippet['local_path'] = local_path

    def _concat_snippets(self, filepath):
        files = [s['local_path'] for s in self.snippets]
        command = [
                settings.LOHROTHEK_FFMPEG,
                '-y', '-i',
                'concat:{}'.format("|".join(files)),
                '-af', 'volume=9dB',
                '-ab', '192k',
                filepath]
        subprocess.run(command)

    def _trim_audiofile(self, filepath):
        start_sec = (self.start_time - parse_datetime(self.snippets[0]['start_time'])) \
                .total_seconds()
        length_sec = (self.end_time - self.start_time).total_seconds()
        command = [
                settings.LOHROTHEK_FFMPEG,
                '-y', '-i',
                filepath,
                '-acodec', 'copy',
                '-ss', str(start_sec),
                '-t', str(length_sec),
                self.audiofile.name]
        subprocess.run(command)


def get_audio_metadata(path_or_obj) -> AudioMetadata:
    meta = mutagen.File(path_or_obj)
    container = None
    codec = None
    sample_rate = None
    length = int(meta.info.length)

    try:
        file_path = path_or_obj.path
    except AttributeError:
        file_path = str(path_or_obj)

    for fmt in EncodingFormat:  # type: EncodingFormat
        if fmt.is_format(meta.mime, fmt.info):
            container = fmt.info.container
            codec = fmt.info.codec
            break

    try:
        bitrate = meta.info.bitrate
    except AttributeError:
        # fallback to a simple (file size / duration) calculation
        # st_size is in bytes, but we want a *bit*rate
        # so we have to multiply it by 8
        bitrate = os.stat(file_path).st_size * 8 / length

    try:
        sample_rate = meta.info.sample_rate
    except AttributeError:
        # unfortunately mutagen will fail to provide a
        # sample rate for some formats (e.g. OPUS)
        pass

    return AudioMetadata(
        container,
        codec,
        bitrate,
        meta.info.channels,
        length,
        sample_rate
    )


class TranscodeError(Exception):
    def __init__(self, encoder_info, message) -> None:
        self.encoder_info = encoder_info
        self.message = message


def transcode_file(src: FilePath, dest: FilePath,
                   ffmpeg_args: Sequence[str], encoder_info: EncoderInfo):
    pre_args = [settings.LOHROTHEK_FFMPEG, '-y', '-loglevel', 'error', '-i', str(src)]
    post_args = [str(dest)]
    args = pre_args + list(ffmpeg_args) + post_args
    proc = subprocess.run(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)

    try:
        proc.check_returncode()
    except subprocess.CalledProcessError as exc:
        raise TranscodeError(encoder_info, proc.stderr.decode('utf-8')) from exc
    return EncodedFile(dest, encoder_info)


def transcode_file_aac(src: FilePath, dest: FilePath):
    flags = ('-movflags', '+faststart', '-f', 'mp4')

    try:
        # libfdk_aac, though open-source, is patent-encumbered and not included in Debian.
        # It provides better quality than the native ffmpeg AAC encoder, so we should use
        # it if it’s available but will likely fail.
        return transcode_file(src, dest, flags + ('-c:a', 'libfdk_aac', '-vbr', '3'),
                              EncodingFormat.MP4_AAC.info)
    except TranscodeError:
        # Variable bitrate in the default AAC encoder is discouraged by the
        # ffmpeg developers, so we use a constant bitrate instead.
        # see: https://trac.ffmpeg.org/wiki/Encode/AAC#NativeFFmpegAACEncoder
        return transcode_file(src, dest, flags + ('-c:a', 'aac', '-b:a', '96k'),
                              EncodingFormat.MP4_AAC.info)


def transcode_file_opus(src: FilePath, dest: FilePath):
    return transcode_file(src, dest, ('-f', 'oga', '-c:a', 'libopus', '-b:a', '64k'),
                          EncodingFormat.OGG_OPUS.info)


def transcode_file_mp3(src: FilePath, dest: FilePath):
    return transcode_file(src, dest, ('-f', 'mp3', '-c:a', 'libmp3lame', '-q:a', '6'),
                          EncodingFormat.MP3.info)


class EncodingFormat(enum.Enum):
    """
    Each instance of EncodingFormat is created with
      * basic encoder information containing the
        * format name
        * container mime type
        * codec mime type
        * file suffix
      * a function that accepts a source and destination argument
        and will use it to transcode the file into the format specified
        by the encoder information
      * and a test function that accepts a list of mime types and the
        encoder info in order to determine if the mime types correspond
        with the format
    """

    MP4_AAC = (
        EncoderInfo('AAC', 'audio/mp4', 'audio/aac', 'aac'),
        transcode_file_aac,
        lambda mime, info: info.container in mime and info.codec in mime)
    OGG_OPUS = (
        EncoderInfo('OPUS', 'audio/ogg', 'audio/opus', 'opus'),
        transcode_file_opus,
        lambda mime, info: info.container in mime and 'audio/ogg; codecs=opus' in mime)
    MP3 = (
        EncoderInfo('MP3', None, 'audio/mpeg', 'lq.mp3'),
        transcode_file_mp3,
        lambda mime, info: info.codec in mime)

    def __init__(self, info: EncoderInfo, encoder: callable, is_format: callable) -> None:
        self.info = info
        self.encoder = encoder
        self.is_format = is_format

    def __call__(self, *args, **kwargs):
        return self.encoder(*args, **kwargs)


def transcode(src: FilePath, formats: Iterable[EncodingFormat]) -> Iterator[EncodedFile]:
    for fmt in formats:
        dest = '{}.{}'.format(str(src), fmt.info.file_suffix)
        try:
            yield fmt(src, dest)
        except TranscodeError as exc:
            logger.error(
                'could not transcode file "{}" to {} format'.format(
                    src, exc.encoder_info.format_name),
                extra=dict(stderr=exc.message), exc_info=exc)
            raise


def resolve_formats(specs: Iterable[Union[str, EncodingFormat]]) -> List[EncodingFormat]:
    """
    Resolves EncodingFormat instances by trying to extract str values
    as attribute from EncodingFormat.

    :param specs: an iterable of EncodingFormat instances or
                  strings referring to an EncodingFormat
    :return: a list of EncodingFormat
    """
    return [
        (getattr(EncodingFormat, spec) if isinstance(spec, str) else spec)
        for spec in specs
    ]
