from io import BytesIO

from django.urls import reverse

from audiothek.models import Audiofile, RadioReport, RadioReportType
from core.tests import TestCase
from shutil import copyfile, rmtree
import os

TEST_FILE = 'testme.mp3'


class ReportMixin(TestCase):
    def setUp(self):
        super().setUp()
        self.report = RadioReport(
                title='Test Title', license=self.license)
        self.report.save()


class AudiofileMixin(ReportMixin):
    def setUp(self):
        super().setUp()

        if not os.path.exists('media'):
            os.mkdir('media')

        if not os.path.exists(os.path.join('media', 'audio')):
            os.mkdir(os.path.join('media', 'audio'))

        copyfile('test.mp3', os.path.join('media', TEST_FILE))

        self.audiofile = Audiofile.objects.create(
                report=self.report, audiofile=TEST_FILE, creator=self.user)

    def tearDownModule(self):
        os.remove(os.path.join('media', TEST_FILE))
        rmtree('media')


class ReportTests(ReportMixin, TestCase):
    def test_report_creation(self):
        create_url = reverse('create-report')
        list_url = reverse('list-reports')
        test_title = 'Create Test Title'

        r = self.client.get(list_url)
        self.assertNotContains(r, test_title)
        self.assertContains(r, 'href="{}"'.format(create_url))

        r = self.client.get(create_url)
        self.assertEqual(r.status_code, 200)

        r = self.client.post(create_url, {
            'title': test_title, 'tags': 'Tag', 'license': self.license.id,
            'type': RadioReportType.report.name, 'editor': self.user.id})
        q = RadioReport.objects.filter(title=test_title)
        self.assertTrue(q.exists())
        self.assertRedirects(r, q.get().get_absolute_url())

        r = self.client.get(list_url)
        self.assertContains(r, test_title)

    def test_report_update(self):
        list_url = reverse('list-reports')
        detail_url = self.report.get_absolute_url()
        update_url = reverse('update-report', args={self.report.pk})
        updated_test_title = 'Update Test Title'

        r = self.client.get(detail_url)
        self.assertContains(r, update_url)

        r = self.client.get(update_url)
        self.assertEqual(r.status_code, 200)

        r = self.client.post(update_url, {
            'title': updated_test_title, 'tags': 'Tag', 'license': self.license.id,
            'type': RadioReportType.report.name, 'editor': self.user.id})
        self.assertRedirects(r, detail_url)

        r = self.client.get(list_url)
        self.assertContains(r, updated_test_title)

    def test_report_deletion(self):
        list_url = reverse('list-reports')
        update_url = reverse('update-report', args={self.report.pk})
        delete_url = reverse('delete-report', args={self.report.pk})

        r = self.client.get(update_url)
        self.assertContains(r, delete_url)

        r = self.client.get(delete_url)
        self.assertEqual(r.status_code, 200)

        r = self.client.post(delete_url)
        self.assertRedirects(r, list_url)

        r = self.client.get(list_url)
        self.assertNotContains(r, self.report.title)


class AudiofileTests(AudiofileMixin, TestCase):
    def test_file_creation(self):
        create_url = reverse('create-audiofile', args=(self.report.pk,))
        list_url = self.report.get_absolute_url()
        test_file = BytesIO(bytes.fromhex(
            '4449 0433 0000 0000 2300 5354 4553 0000 '
            '0f00 0000 4c03 7661 3566 2e37 3635 312e '
            '3130 0000 0000 0000 0000 0000 ff00 b0fb '
            '0000 0000 0000 0000 0000 0000 0000 0000 '))

        r = self.client.get(list_url)

        self.assertNotContains(r, test_file)
        self.assertContains(r, 'href="{}"'.format(create_url))

        r = self.client.get(create_url)

        self.assertEqual(r.status_code, 200)

        # FIXME: create proper mp3 file
        # test_file_title = 'Create Test Title'
        # r = self.client.post(create_url, {'audiofile': test_file, 'title': test_file_title})
        # self.assertRedirects(r, list_url)
        #
        # r = self.client.get(list_url)
        # self.assertContains(r, test_file_title)

    def test_file_update(self):
        list_url = self.report.get_absolute_url()
        update_url = reverse('update-audiofile', args={self.audiofile.pk})
        updated_test_title = 'Updated Test Title'

        r = self.client.get(list_url)
        self.assertContains(r, update_url)

        r = self.client.get(update_url)
        self.assertEqual(r.status_code, 200)

        r = self.client.post(update_url, {
            'title': updated_test_title, 'audiofile': self.audiofile.audiofile})
        self.assertRedirects(r, list_url)

        r = self.client.get(list_url)

        self.assertContains(r, updated_test_title)

    def test_file_deletion(self):
        list_url = self.report.get_absolute_url()
        update_url = reverse('update-audiofile', args={self.audiofile.pk})
        delete_url = reverse('delete-audiofile', args={self.audiofile.pk})

        r = self.client.get(update_url)
        self.assertContains(r, delete_url)

        r = self.client.get(delete_url)
        self.assertEqual(r.status_code, 200)

        r = self.client.post(delete_url)
        self.assertRedirects(r, list_url)

        r = self.client.get(list_url)

        self.assertNotContains(r, self.audiofile.audiofile)
