from dal import autocomplete
from django_filters.views import FilterView
from django.shortcuts import get_object_or_404, redirect
from django.urls import reverse_lazy
from django.views.generic import CreateView, DetailView, UpdateView, DeleteView
from django.views.generic.edit import ModelFormMixin, FormMixin
from django.contrib import messages
from django.contrib.messages.views import SuccessMessageMixin

from core.models import License, ContentTag
from audiothek.forms import AudiofileFromRecordingForm, RadioReportForm, \
                            AudiofileForm, RadioReportQuickForm, RadioReportRoleForm
from audiothek.models import RadioReport, Audiofile, RadioReportRole, RadioReportType
from audiothek.filters import RadioReportFilter
from audiothek.processing import get_audio_metadata
from program.models import Broadcast, BroadcastItem
from django.urls.base import reverse
from program.filters import BroadcastFilter
from moderation.filters import ModerationFilter
from moderation.models import Moderation
from django.views.generic.base import View
from actstream.signals import action
from activity import verbs
from actstream.models import Action, any_stream
from django.utils import timezone
from django.utils.safestring import mark_safe


class ReportDetail(View):

    def get(self, request, *args, **kwargs):
        view = ReportDisplay.as_view()
        return view(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        view = ReportRoleCreate.as_view()
        return view(request, *args, **kwargs)


class ReportDisplay(DetailView):
    model = RadioReport

    def get_context_data(self, **kwargs):
        context = super(ReportDisplay, self).get_context_data(**kwargs)

        initial = {
            'user': self.request.user,
            'name': RadioReportRole.Name.every.name,
            }
        context['formroles'] = RadioReportRoleForm(initial=initial)

        report_improve = []
        if self.object.roles.count() == 0 and not self.object.externalcreators:
            report_improve.append('Gib mindestens eine*n internen oder externen Urheber*innen an.')
        if self.object.tags.count() == 0:
            report_improve.append('Gib Schlagworte an.')
        if self.object.categories.count() == 0:
            report_improve.append('Weise mindestens eine Kategorie zu.')
        if not self.object.description:
            report_improve.append('Gib eine Beschreibung an.')
        if report_improve:
            messages.add_message(self.request, messages.INFO,
                                 mark_safe('Der Beitrag könnte noch verbessert werden:\
                                 <ul><li>%(msgs)s</li></ul>' %
                                 {'msgs': '</li><li>'.join(report_improve)})) # noqa
        return context


class ReportRoleCreate(CreateView, FormMixin):
    template_name = 'audiothek/radioreport_detail.html'
    form_class = RadioReportRoleForm
    model = RadioReportRole

    def form_valid(self, form):
        rpk = self.kwargs.get('pk')
        report = get_object_or_404(RadioReport, pk=rpk)

        self.object = form.save(commit=False)
        self.object.report = report

        existingRoles = RadioReportRole.objects.filter(report=report)

        for exR in existingRoles:
            if exR.user == self.object.user and exR.name == self.object.name:
                messages.add_message(self.request, messages.ERROR,
                                     "Nutzer*in [%(u)s] wurde für den Beitrag bereits \
                                     die Rolle [%(r)s] zugewiesen." %
                                     {'u': self.object.user,
                                      'r': RadioReportRole.Name[self.object.name]})
                return redirect(self.get_success_url())

        messages.add_message(self.request, messages.SUCCESS,
                             "Nutzer*in [%(u)s] wurde mit Rolle [%(r)s] für den Beitrag erstelt."
                             % {'u': self.object.user,
                                'r': RadioReportRole.Name[self.object.name]})

        return CreateView.form_valid(self, form)

    def get_success_url(self):
        return reverse('show-report', kwargs={'pk': self.object.report.pk})


class ReportRoleDelete(DeleteView):
    model = RadioReportRole

    def get_success_url(self):
        return reverse('show-report', kwargs={'pk': self.object.report.pk})


def change_reportrole_visibility(request, role_pk):
    role = get_object_or_404(RadioReportRole, pk=role_pk)
    role.public = not role.public
    role.save()
    messages.add_message(request, messages.SUCCESS,
                         "Sichtbarkeit for Rolle  [%(r)s] zu Urheber*in [%(u)s] geändert."
                         % {'u': role.user,
                            'r': RadioReportRole.Name[role.name]})
    return redirect(reverse('show-report', kwargs={'pk': role.report.pk}))


class ReportList(FilterView):
    filterset_class = RadioReportFilter
    context_object_name = 'reports'
    paginate_by = 10
    template_name = 'audiothek/radioreport_list.html'


class ReportCreate(CreateView):
    model = RadioReport
    form_class = RadioReportForm

    def get_context_data(self, **kwargs):
        if 'broadcast' in self.kwargs:
            kwargs['broadcast'] = Broadcast.objects.get(pk=self.kwargs.get('broadcast'))
            messages.info(self.request, "Dieser Beitrag wird bei Erstellung \
            der Sendung %(broadcast)s zugeordnet" % {"broadcast": kwargs['broadcast']})
        context = super(ReportCreate, self).get_context_data(**kwargs)
        return context

    def get_initial(self):
        license = License.objects.filter(default=True).first()
        type = RadioReportType.report
        publication_time = timezone.now()
        return {'license': license, 'type': type, 'publication_time': publication_time}

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.creator = self.request.user
        self.object.creation_time = timezone.now()

        if 'broadcast' in self.kwargs:
            broadcast = Broadcast.objects.get(pk=self.kwargs.get('broadcast'))
            bcitem = BroadcastItem()
            bcitem.report = self.object
            bcitem.broadcast = broadcast
            bcitem.save()

        action.send(self.request.user, verb=verbs.CREATED_RADIOREPORT, target=self.object)

        return super().form_valid(form)


class ReportQuickCreate(CreateView):
    model = RadioReport
    form_class = RadioReportQuickForm
    template_name = 'audiothek/radioreport_quickform.html'

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        rep = RadioReport()
        rep.save()
        kwargs['instance'] = rep
        return kwargs

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.creator = self.request.user
        self.object.creation_time = timezone.now()

        file = Audiofile(report=self.object,
                         creator=self.request.user,
                         audiofile=self.request.FILES['audiofile'])
        file.save()

        action.send(self.request.user, verb=verbs.CREATED_RADIOREPORT, target=self.object)
        return super().form_valid(form)


class ReportUpdate(UpdateView):
    model = RadioReport
    form_class = RadioReportForm

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.last_editor = self.request.user
        self.object.last_edit_time = timezone.now()

        # retrieve instance actions
        instance_actions = any_stream(self.object)

        # overwrite latest edit action by same user within 120s
        if instance_actions \
            and (timezone.now() - instance_actions[0].timestamp).total_seconds() < 120 \
                and instance_actions[0].actor_object_id == self.request.user.pk \
                and instance_actions[0].verb == verbs.CHANGED_RADIOREPORT:
                    # update last edit action to current time
                    act = Action.objects.get(pk=instance_actions[0].pk)
                    act.timestamp = timezone.now()
                    act.save()
        else:
            action.send(self.request.user, verb=verbs.CHANGED_RADIOREPORT, target=self.object)
        return super().form_valid(form)


class ReportDelete(DeleteView):
    model = RadioReport
    success_url = reverse_lazy('list-reports')

    def delete(self, request, *args, **kwargs):
        obj = self.get_object()
        action.send(self.request.user, verb=verbs.DELETED_RADIOREPORT, target=obj)
        return DeleteView.delete(self, request, *args, **kwargs)


def quick_add_broadcast_to_report(request, report_pk, broadcast_pk):

    broadcast = get_object_or_404(Broadcast, pk=broadcast_pk)
    report = get_object_or_404(RadioReport, pk=report_pk)

    bcitem = BroadcastItem()
    bcitem.broadcast = broadcast
    bcitem.report = report
    bcitem.save()

    messages.add_message(request, messages.SUCCESS,
                         "Der Sendung %(b)s wurde dem Beitrag %(r)s hinzugefügt." %
                         {'r': report, 'b': broadcast})

    return redirect(reverse('show-report', kwargs={'pk': report.pk}))


class RadioReportBroadcastSelect(FilterView):
    filterset_class = BroadcastFilter
    context_object_name = 'broadcasts'
    paginate_by = 10
    template_name = 'audiothek/radioreport_select_broadcast.html'

    def get_context_data(self, **kwargs):
        context = super(RadioReportBroadcastSelect, self).get_context_data(**kwargs)
        context['report'] = get_object_or_404(RadioReport, pk=self.kwargs.get('report_pk'))
        return context


class ReportModerationSelect(FilterView):
    filterset_class = ModerationFilter
    context_object_name = 'moderations'
    paginate_by = 10
    template_name = 'audiothek/radioreport_select_moderation.html'

    def get_context_data(self, **kwargs):
        context = super(ReportModerationSelect, self).get_context_data(**kwargs)
        context['report'] = get_object_or_404(RadioReport, pk=self.kwargs.get('report_pk'))
        return context


def add_moderation_to_report(request, report_pk, moderation_pk):

    moderation = get_object_or_404(Moderation, pk=moderation_pk)
    report = get_object_or_404(RadioReport, pk=report_pk)

    moderation.report = report
    moderation.save()

    messages.add_message(request, messages.SUCCESS,
                         "Der Moderation %(m)s wurde dem Beitrag %(r)s hinzugefügt." %
                         {'m': moderation, 'r': report})

    return redirect(reverse('show-report', kwargs={'pk': report.pk}))


def remove_moderation_from_report(request, report_pk, moderation_pk):

    moderation = get_object_or_404(Moderation, pk=moderation_pk)
    report = get_object_or_404(RadioReport, pk=report_pk)

    moderation.report = None
    moderation.save()

    messages.add_message(request, messages.SUCCESS,
                         "Der Moderation %(m)s wurde vom Beitrag %(r)s getrennt." %
                         {'m': moderation, 'r': report})

    return redirect(reverse('show-report', kwargs={'pk': report.pk}))


class AudiofileCreate(SuccessMessageMixin, CreateView):
    model = Audiofile
    form_class = AudiofileForm
    success_message = "Audiodatei wurde erfolgreich erstellt."

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        report = get_object_or_404(RadioReport, pk=self.kwargs.get('report_pk'))
        kwargs['instance'] = Audiofile(report=report, creator=self.request.user)
        return kwargs

    def form_valid(self, form):
        self.object = form.save(commit=False)
        get_audio_metadata(self.object.audiofile).assign_to(self.object)
        self.object.save()
        messages.success(self.request, self.success_message % self.object.__dict__)
        action.send(self.request.user, verb=verbs.CREATED_AUDIOFILE,
                    action_object=self.object, target=self.object.report)
        return super(ModelFormMixin, self).form_valid(form)


class AudiofileFromRecordingCreate(CreateView):
    form_class = AudiofileFromRecordingForm
    template_name = 'audiothek/audiofile_recording_create.html'

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        report = get_object_or_404(RadioReport, pk=self.kwargs.get('report_pk'))
        kwargs['instance'] = Audiofile(report=report, creator=self.request.user)
        return kwargs


class AudiofileUpdate(SuccessMessageMixin, UpdateView):
    model = Audiofile
    form_class = AudiofileForm
    success_message = "Audiodatei wurde erfolgreich aktualisiert."

    def form_valid(self, form):
        self.object = form.save(commit=False)
        get_audio_metadata(self.object.audiofile).assign_to(self.object)
        self.object.save()
        messages.success(self.request, self.success_message % self.object.__dict__)
        action.send(self.request.user, verb=verbs.CHANGED_AUDIOFILE,
                    action_object=self.object, target=self.object.report)
        return super(ModelFormMixin, self).form_valid(form)


class AudiofileDelete(SuccessMessageMixin, DeleteView):
    model = Audiofile
    success_message = "Audiodatei wurde erfolgreich gelöscht."

    def delete(self, request, *args, **kwargs):
        obj = self.get_object()
        messages.success(self.request, self.success_message % obj.__dict__)
        action.send(self.request.user, verb=verbs.DELETED_AUDIOFILE,
                    action_object=obj, target=obj.report)
        return super(AudiofileDelete, self).delete(request, *args, **kwargs)

    def get_success_url(self):
        return self.object.report.get_absolute_url()


def move_audiofile(request, audiofile_pk, action):

    def top():
        items = Audiofile.objects.filter(report=item.report).order_by('position')
        if item.position == 1:
            messages.add_message(request, messages.WARNING,
                                 "Der gewählte Eintrag ist bereits am Anfang der Liste")
            return

        for i in items:
            if i.position < item.position:
                i.position += 1
                i.save()
        item.position = 1
        item.save()
        return

    def up():
        if item.position == 1:
            messages.add_message(request, messages.WARNING,
                                 "Der gewählte Eintrag ist bereits am Anfang der Liste")
            return

        upper_item = Audiofile.objects.filter(report=item.report)\
            .filter(position=item.position - 1).get()
        upper_item.position += 1
        upper_item.save()
        item.position -= 1
        item.save()
        return

    def down():
        try:
            lower_item = Audiofile.objects.filter(report=item.report)\
                .filter(position=item.position + 1).get()
        except BroadcastItem.DoesNotExist:
            messages.add_message(request, messages.WARNING,
                                 "Der gewählte Eintrag ist bereits am Ende der Liste")
            return

        lower_item.position -= 1
        lower_item.save()
        item.position += 1
        item.save()
        return

    def bottom():
        items = Audiofile.objects.filter(report=item.report).order_by('position')

        if item.position == len(items):
            messages.add_message(request, messages.WARNING,
                                 "Der gewählte Eintrag ist bereits am Ende der Sequenz")

        for i in items:
            if i.position > item.position:
                i.position -= 1
                i.save()
        item.position = len(items)
        item.save()
        return

    switcher = {
        'top': top,
        'up': up,
        'down': down,
        'bottom':  bottom
    }

    item = get_object_or_404(Audiofile, pk=audiofile_pk)

    if action not in switcher.keys():
        messages.add_message(request, messages.WARNING,
                             "Der Wert {0} ist nicht gültig für den Parameter move").format(action)
    else:
        func = switcher.get(action, "nothing")
        func()
    return redirect(
        reverse('show-report', kwargs={'pk': item.report.pk}))


class ContentTagAutocomplete(autocomplete.Select2QuerySetView):

    def get_queryset(self):
        tags = ContentTag.objects.all()
        if self.q:
            return tags.filter(name__icontains=self.q)
        # temporarily disable count order due to later usage in broadcasts too
        # return tags.annotate(popularity=Count('taggit_taggeditem_items')) \
        #    .order_by('-popularity')[:10]
