import re
import sys

from django.conf import settings
from django.http import FileResponse, HttpResponseRedirect
from django.views.debug import technical_500_response

from core.http import HttpResponseSendfile


class LoginRequiredMiddleware:
    ANON_ROUTES = re.compile(r'^\/(?:api)\/')

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if self.should_redirect_to_login(request):
            return HttpResponseRedirect(settings.LOGIN_URL)

        return self.get_response(request)

    @classmethod
    def should_redirect_to_login(cls, request):
        # If the request path matches any of the anon-routes
        # no authentication should be forced
        if cls.ANON_ROUTES.match(request.path) is not None:
            return False

        # Static and media routes are handled by the web server in production.
        # Forcing logins on them will only work with the development server and
        # might lead to the impression that these urls are login protected.
        if request.path.startswith(settings.STATIC_URL) or \
                request.path.startswith(settings.MEDIA_URL):
            return False

        # If the requested URL is already the login url there is no point in
        # redirecting the request to the same location.
        if request.path == settings.LOGIN_URL:
            return False

        # Only force authentication, if the user has not been authenticated yet
        return not request.user.is_authenticated


class DevelopmentSendFileMiddleware:
    """
    Middleware that converts HttpResponseSendfile into a plain old FileResponse.
    HttpResponseSendfile carries X-Sendfile and X-Accel-Redirect headers meant
    for consumption by application servers like uWSGI or web services like NGINX.

    The development server does not automatically convert those responses into
    FileResponses so this middleware does it instead. It MUST only be used
    if DEBUG is enabled and will refuse to work otherwise.
    """
    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        response = self.get_response(request)

        if settings.DEBUG is False:
            # Refuse to to work if DEBUG is disabled.
            # It might be a good idea to check for this earlier (ideally with check run by
            # Django’s system check framework) but the way settings are structured right now
            # it’s hard to enable this middleware and have it not fail when running tests.
            # Just don’t add it when running in production!
            return response

        if isinstance(response, HttpResponseSendfile):
            response_file = response.file_field.open()
            if response_file is None:
                # this is a weird behaviour that happens with
                # cached image files that have already been created
                # in a previous request
                response_file = response.file_field.file.open()
            return FileResponse(response_file)
        else:
            return response


class UserBasedExceptionMiddleware(object):

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        return self.get_response(request)

    def process_exception(self, request, exception):
        if request.user.is_superuser:
            return technical_500_response(request, *sys.exc_info())
