from enum import Enum

from django.conf import settings
from django.contrib.auth.models import User
from django.db import models
from django.utils import timezone
from django.db.models.signals import post_save
from django.dispatch import receiver
from imagekit.models import ImageSpecField
from imagekit.processors import ResizeToFill
from django.urls.base import reverse
from taggit.models import TagBase
from django.utils.translation import ugettext_lazy as _


def django_enum(cls):
    # decorator needed to enable enums in django templates
    cls.do_not_call_in_templates = True
    return cls


class Image(models.Model):
    image_file = models.ImageField()
    image_preview = ImageSpecField(source='image_file', processors=[ResizeToFill(100, 100)])


class License(models.Model):
    name = models.CharField(max_length=200)
    identifier = models.CharField(max_length=50, help_text='SPDX License Identifier')
    url = models.URLField()
    default = models.BooleanField(default=False)

    def __str__(self):
        return self.name


class SoftDeleteManager(models.Manager):
    """modified manager is necessary to handle soft-delete"""
    def get_queryset(self):
        return super().get_queryset().filter(deletion_time=None)


class SoftDeleteModel(models.Model):
    """add this to your class to get Soft-Delete-Capability"""
    objects = SoftDeleteManager()
    admin_objects = models.Manager()
    deletion_time = models.DateTimeField('Löschung (Datum/Uhrzeit)', null=True, blank=True)

    class Meta:
        abstract = True

    def delete(self, **kwargs):
        """override delete() for soft deletion"""
        self.deletion_time = timezone.now()
        self.save()

    def hard_delete(self):
        """call this if you want the object actual removed from the database"""
        super().delete(self)


class Profile(models.Model):
    user = models.OneToOneField('auth.User', on_delete=models.CASCADE)
    nickname = models.CharField('Spitzname', max_length=100, blank=True)
    image = models.ImageField('Bild', blank=True)
    about = models.TextField('Über mich', blank=True)

    def get_display_name(self):
        return self.nickname or self.user.username


class Department(models.Model):
    name = models.CharField(max_length=255)
    programs = models.ManyToManyField('program.RadioProgram', related_name='departments',
                                      blank=True)

    class Meta:
        ordering = ['name']

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('show-department', args=[self.pk])


class DepartmentRole(models.Model):

    @django_enum
    class Name(Enum):
        voluntary = "Ehrenamt"
        instructor = "Fachanleitung"
        voluntary_service = "Freiwilligendienst"
        employee = "Mitarbeit"
        trainee = "Praktikum"
        head = "Redaktionsleitung"

        def __repr__(self):
            return self.name

        def __str__(self):
            return self.value

    name = models.CharField(max_length=31, choices=[(role.name, role.value) for role in Name])
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.PROTECT,
                             related_name="departmentroles")

    start_date = models.DateField(blank=True, null=True)
    end_date = models.DateField(blank=True, null=True)

    department = models.ForeignKey('Department', on_delete=models.CASCADE,
                                   null=True, blank=True, related_name='roles')

    class Meta:
        ordering = ['name', 'department']

    def __str__(self):
        return "{} @ {}: {} ({}–{})".format(self.user, self.department,
                                            self.Name[self.name], self.start_date, self.end_date)


class ContentTag(TagBase):

    description = models.TextField('Beschreibung', null=True)

    class Meta:
        verbose_name = _("Tag")
        verbose_name_plural = _("Tags")


@receiver(post_save, sender=User)
def create_user_profile(sender, instance, created, **kwargs):
    if created:
        Profile.objects.create(user=instance)
