from datetime import datetime, timedelta
from django.db.models import Q
from django.shortcuts import render, get_object_or_404
from django.views.generic import UpdateView
from django_filters.views import FilterView

from audiothek.models import RadioReport
from core.models import Profile, Department
from core.filters import DepartmentFilter, DepartmentRoleFilter, UserFilter
from program.models import Broadcast
from program.querysets import query_broadcasts_of_day
from django.urls.base import reverse_lazy
from django.views.generic.detail import DetailView
from django.contrib.auth.models import User
from dal import autocomplete


def index(request):

    context = {}

    context['radioreports_last'] = RadioReport.objects.order_by('-production_date')[:10]

    today = datetime.now()
    tomorrow = today + timedelta(days=1)

    context['broadcasts_today'] = query_broadcasts_of_day(
        Broadcast.objects.get_queryset(), today, False)
    context['broadcasts_tomorrow'] = query_broadcasts_of_day(
        Broadcast.objects.get_queryset(), tomorrow, False)

    return render(request, 'core/index.html', context)


class ProfileUpdateView(UpdateView):
    model = Profile
    fields = ('nickname', 'image', 'about')
    success_url = reverse_lazy('home')

    def get_object(self):
        return self.request.user.profile


class DepartmentDetail(FilterView):
    filterset_class = DepartmentRoleFilter
    context_object_name = 'members'
    paginate_by = 10
    template_name = 'core/department_detail.html'

    def get_context_data(self, *, object_list=None, **kwargs):
        context = FilterView.get_context_data(self, **kwargs)
        pk = self.kwargs['pk']
        context['department'] = get_object_or_404(Department, pk=pk)
        return context

    def get_filterset_kwargs(self, filterset_class):
        kwargs = super(DepartmentDetail, self).get_filterset_kwargs(filterset_class)
        pk = self.kwargs['pk']
        department = get_object_or_404(Department, pk=pk)
        kwargs['department'] = department
        return kwargs


class DepartmentList(FilterView):
    filterset_class = DepartmentFilter
    context_object_name = 'departments'
    paginate_by = 10
    template_name = 'core/department_list.html'


class UserDetail(DetailView):
    model = User
    template_name = 'core/user_detail.html'


class UserList(FilterView):
    filterset_class = UserFilter
    context_object_name = 'users'
    paginate_by = 10
    template_name = 'core/user_list.html'


class UserAutocomplete(autocomplete.Select2QuerySetView):

    def get_queryset(self):
        if not self.request.user.is_authenticated:
            return User.objects.none()

        qs = User.objects.all()

        if self.q:
            qs = qs.filter(Q(username__icontains=self.q) |
                           Q(first_name__icontains=self.q) |
                           Q(last_name__icontains=self.q) |
                           Q(profile__nickname__icontains=self.q))
        return qs

    def get_result_label(self, item):
        return "{} ({} {})".format(item.username, item.first_name, item.last_name)
