from django.db import models
from django.urls.base import reverse
from django.utils import timezone
from django.conf import settings


class Moderation(models.Model):
    name = models.CharField('Name', max_length=255)
    note = models.CharField('Notiz', max_length=255, blank=True, null=True)
    report = models.ForeignKey('audiothek.RadioReport', on_delete=models.SET_NULL,
                               related_name='moderations', blank=True, null=True)
    length = models.TimeField('Länge', blank=True, null=True)
    creation_time = models.DateTimeField('erstellt', default=timezone.now)
    creator = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.PROTECT, null=True,
                                related_name='moderation_creator')
    last_edit_time = models.DateTimeField('zuletzt bearbeitet', blank=True, null=True)
    last_editor = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.PROTECT, null=True,
                                    related_name='moderation_last_editor')

    def __str__(self):
        if self.report:
            return self.report.title
        return self.name

    def get_absolute_url(self):
        return reverse('show-moderation', args=[self.pk])


class ModerationItem(models.Model):
    position = models.PositiveSmallIntegerField(default=1)
    moderation = models.ForeignKey(
        Moderation, on_delete=models.CASCADE, related_name='items', null=True)

    def get_absolute_url(self):
        return reverse('show-moderation', args=[self.moderation.pk])

    def save(self, *args, **kwargs):
        if not self.pk:
            existingItems = ModerationItem.objects.filter(moderation=self.moderation)
            self.position = existingItems.count()+1
        super(ModerationItem, self).save()

    def delete(self):
        existHigherPos = ModerationItem.objects.filter(
            moderation=self.moderation).filter(position__gt=self.position)
        if existHigherPos:
            for p in existHigherPos:
                p.position -= 1
                p.save()
        super(ModerationItem, self).delete()

    class Meta:
        ordering = ['position']


class ModerationItemSpeak(models.Model):
    content = models.TextField('Sprechtext')
    item = models.OneToOneField(ModerationItem,
                                on_delete=models.CASCADE, related_name="speak")

    def save(self, *args, **kwargs):
        if self.pk is not None:
            orig = ModerationItemSpeak.objects.get(pk=self.pk)
            if orig.content != self.content:
                moderation = self.item.moderation
                moderation.edited_datetime = timezone.now()
                moderation.save()
        super(ModerationItemSpeak, self).save(*args, **kwargs)

    def get_absolute_url(self):
        return reverse('show-moderation', args=[self.item.moderation.pk])


class ModerationItemSystem(models.Model):
    folder = models.CharField('Speicherort', max_length=255, blank=True, null=True)
    name = models.CharField('Name', max_length=255)
    audiofile = models.ForeignKey('audiothek.Audiofile', on_delete=models.SET_NULL,
                                  blank=True, null=True)
    item = models.OneToOneField(ModerationItem,
                                on_delete=models.CASCADE, related_name="system")
    length = models.TimeField('Länge', blank=True, null=True)

    def get_absolute_url(self):
        return reverse('show-moderation', args=[self.item.moderation.pk])

    def save(self, *args, **kwargs):
        if self.pk is not None:
            orig = ModerationItemSystem.objects.get(pk=self.pk)
            if orig.name != self.name or orig.folder != self.folder \
               or orig.audiofile != self.audiofile:
                moderation = self.item.moderation
                moderation.edited_datetime = timezone.now()
                moderation.save()

        super(ModerationItemSystem, self).save(*args, **kwargs)

    def delete(self):
        item = ModerationItem.objects.filter(system=self)
        if item:
            item.delete()
        super(ModerationItemSystem, self).delete()
