from django.shortcuts import get_object_or_404, redirect
from django.contrib.messages.views import SuccessMessageMixin
from django.views.generic.edit import CreateView, UpdateView, DeleteView,\
    ModelFormMixin
from moderation.models import Moderation, ModerationItem, ModerationItemSystem,\
    ModerationItemSpeak
from datetime import datetime, timedelta
from django.views.generic.detail import DetailView
from django.contrib import messages
import textwrap
from moderation.forms import ModerationItemSystemForm, ModerationForm
from django.urls.base import reverse, reverse_lazy
from audiothek.models import Audiofile, RadioReport
from django_filters.views import FilterView
from moderation.filters import ModerationFilter
from audiothek.filters import RadioReportFilter
from program.models import Broadcast, BroadcastItem
from program.filters import BroadcastFilter
from django.template.loader import render_to_string
from django.utils import timezone
import time


class ModerationList(FilterView):
    filterset_class = ModerationFilter
    context_object_name = 'moderations'
    paginate_by = 10
    template_name = 'moderation/moderation_list.html'


class ModerationCreate(SuccessMessageMixin, CreateView):
    model = Moderation
    fields = {'name', 'note'}
    success_message = "Moderation %(name)s wurde erstellt."

    def dispatch(self, request, *args, **kwargs):

        if 'report' in self.kwargs:
            report = RadioReport.objects.get(pk=self.kwargs.get('report'))
            if report:
                m = Moderation(report=report)
                m.save()
                kwargs['report'] = RadioReport.objects.get(pk=self.kwargs.get('report'))
                messages.success(self.request, "Eine Moderation angelegt und dem Beitrag \
                %(report)s zugeordnet." % {"report": kwargs['report']})
                return redirect('show-moderation', pk=m.pk)

        return super(ModerationCreate, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        if 'broadcast' in self.kwargs:
            kwargs['broadcast'] = Broadcast.objects.get(pk=self.kwargs.get('broadcast'))
            messages.info(self.request, "Diese Moderation wird bei Erstellung \
            der Sendung %(broadcast)s zugeordnet" % {"broadcast": kwargs['broadcast']})
        context = super(ModerationCreate, self).get_context_data(**kwargs)
        return context

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        if 'report' in self.kwargs:
            report = get_object_or_404(RadioReport, pk=self.kwargs.get('report'))
            kwargs['instance'] = Moderation(report=report, name=report.title)
        return kwargs

    def form_valid(self, form):
        self.object = form.save()
        moderation = form.instance
        moderation.creator = self.request.user
        if 'report' in self.kwargs:
            report = get_object_or_404(RadioReport, pk=self.kwargs.get('report'))
            moderation.report = report
        elif 'broadcast' in self.kwargs:
            broadcast = get_object_or_404(Broadcast, pk=self.kwargs.get('broadcast'))
            bcitem = BroadcastItem()
            bcitem.moderation = moderation
            bcitem.broadcast = broadcast
            bcitem.save()
        return super(ModerationCreate, self).form_valid(form)


class ModerationDetail(DetailView):
    model = Moderation


class ModerationPrintDetail(DetailView):
    model = Moderation
    template_name = 'moderation/moderation_print.html'


class ModerationUpdate(SuccessMessageMixin, UpdateView):
    model = Moderation
    form_class = ModerationForm

    def form_valid(self, form):
        self.object = form.save()
        moderation = form.instance
        moderation.last_editor = self.request.user
        moderation.last_edit_time = timezone.now()
        return SuccessMessageMixin.form_valid(self, form)


class ModerationDelete(SuccessMessageMixin, DeleteView):
    model = Moderation
    success_message = "Moderation %(name)s wurde gelöscht."
    success_url = reverse_lazy('list-moderations')


class ModerationSelectReportList(FilterView):
    filterset_class = RadioReportFilter
    context_object_name = 'reports'
    paginate_by = 10
    template_name = 'moderation/moderation_select_report.html'

    def get_context_data(self, **kwargs):
        context = super(ModerationSelectReportList, self).get_context_data(**kwargs)
        context['moderation'] = get_object_or_404(Moderation, pk=self.kwargs.get('moderation_pk'))
        return context


def moderation_remove_report(request, moderation_pk):

    moderation = get_object_or_404(Moderation, pk=moderation_pk)

    report_to_remove = moderation.report

    moderation.report = None
    moderation.name = report_to_remove.title + ' (früher Beitrag %(b)i)' % \
        {'b': report_to_remove.pk}
    moderation.save()
    messages.add_message(request, messages.SUCCESS,
                         "Der Beitrag '%(b)s' wurde von dieser Moderation entfernt."
                         % {'b': report_to_remove})
    messages.add_message(request, messages.INFO,
                         "Der Titel dieser Moderation wurde gesetzt auf den Titel + ID des lösgelösten Beitrag.\
                         Du kannst ihn überschreiben.")

    if moderation.broadcastitems:

        bcitems = BroadcastItem.objects.filter(moderation=moderation)

        for b in bcitems:
            b.report = None
            b.save()

        if bcitems:
            messages.add_message(
                request, messages.INFO,
                render_to_string(
                    'moderation/_moderation_unlink_report_messsage_broadcastitems.html',
                    {'report_to_remove': report_to_remove, 'bcitems': bcitems}))

    return redirect(reverse('show-moderation', kwargs={'pk': moderation.pk}))


def moderation_add_report(request, moderation_pk, report_pk):

    moderation = get_object_or_404(Moderation, pk=moderation_pk)
    report = get_object_or_404(RadioReport, pk=report_pk)

    moderation.name = ""
    moderation.report = report
    moderation.save()
    messages.add_message(request, messages.SUCCESS,
                         "Der Beitrag '%(r)s' wurde mit dieser Moderation verknüpft."
                         % {'r': report})
    messages.add_message(request, messages.INFO, "Der alte Name der Moderation wurde entfernt,\
        da diese jetzt den Namen des Beitrags trägt")

    if moderation.broadcastitems:

        bcitems = BroadcastItem.objects.filter(moderation=moderation)

        for b in bcitems:
            b.report = report
            b.save()

        if bcitems:
            messages.add_message(
                request, messages.INFO,
                render_to_string('moderation/_moderation_link_report_messsage_broadcastitems.html',
                                 {'report_to_add': report, 'bcitems': bcitems}))

    return redirect(reverse('show-moderation', kwargs={'pk': moderation.pk}))


def quick_add_broadcast_to_moderation(request, broadcast_pk, moderation_pk):

    broadcast = get_object_or_404(Broadcast, pk=broadcast_pk)
    moderation = get_object_or_404(Moderation, pk=moderation_pk)

    bcitem = BroadcastItem()
    bcitem.broadcast = broadcast
    bcitem.moderation = moderation
    if moderation.report:
        bcitem.report = moderation.report
    bcitem.save()

    messages.add_message(request, messages.SUCCESS,
                         "Der Sendung %(b)s wurde der Moderation %(m)s zugeordnet ." %
                         {'b': broadcast, 'm': moderation})

    return redirect(reverse('show-moderation', kwargs={'pk': moderation.pk}))


class ModerationSelectBroadcastList(FilterView):
    filterset_class = BroadcastFilter
    context_object_name = 'broadcasts'
    paginate_by = 10
    template_name = 'moderation/moderation_select_broadcast.html'

    def get_context_data(self, **kwargs):
        context = super(ModerationSelectBroadcastList, self).get_context_data(**kwargs)
        context['moderation'] = get_object_or_404(Moderation, pk=self.kwargs.get('moderation_pk'))
        return context


class ModerationItemSpeakCreate(SuccessMessageMixin, CreateView):
    model = ModerationItemSpeak
    fields = {'content'}

    def get_context_data(self, **kwargs):
        kwargs['moderation'] = get_object_or_404(Moderation, pk=self.kwargs.get('pk'))
        context = super(ModerationItemSpeakCreate, self).get_context_data(**kwargs)
        return context

    def form_valid(self, form):
        speakitem = form.instance
        moderation = get_object_or_404(Moderation, pk=self.kwargs.get('pk'))
        item = ModerationItem(moderation=moderation)
        item.save()
        speakitem.item = item
        return super(ModerationItemSpeakCreate, self).form_valid(form)

    def get_success_message(self, cleaned_data):
        return "Sprechteil %(content)s wurde an Position %(pos)s erstellt." \
            % {'content': textwrap.shorten(self.object.content, 24),
               'pos': self.object.item.position}


class ModerationItemSpeakUpdate(SuccessMessageMixin, UpdateView):
    model = ModerationItemSpeak
    fields = {'content'}

    def get_context_data(self, **kwargs):
        kwargs['moderation'] = self.object.item.moderation
        return super(ModerationItemSpeakUpdate, self).get_context_data(**kwargs)

    def get_success_message(self, cleaned_data):
        return "Moderationsteil %(content)s wurde an Position %(pos)s aktualisiert." \
            % {'content': textwrap.shorten(self.object.content, 24),
               'pos': self.object.item.position}


class ModerationItemSystemCreate(SuccessMessageMixin, CreateView):
    model = ModerationItemSystem
    form_class = ModerationItemSystemForm

    def get_context_data(self, **kwargs):
        kwargs['moderation'] = get_object_or_404(Moderation, pk=self.kwargs.get('pk'))
        return super(ModerationItemSystemCreate, self).get_context_data(**kwargs)

    def get_form(self, *args, **kwargs):
        form = super(ModerationItemSystemCreate, self).get_form(*args, **kwargs)
        moderation = get_object_or_404(Moderation, pk=self.kwargs.get('pk'))
        form.fields['audiofile'].choices = [('', '---')] + [
            (a.id, "Audiodatei #%(pos)i (%(length)s) %(audiotitle)s" %
             {'pos': a.position,
              'length': time.strftime('%H:%M:%S', time.gmtime(a.length)),
              'audiotitle': a.title})
            for a in Audiofile.objects.filter(report=moderation.report)]
        return form

    def form_valid(self, form):
        sysitem = form.instance
        moderation = get_object_or_404(Moderation, pk=self.kwargs.get('pk'))
        item = ModerationItem(moderation=moderation)
        item.save()
        sysitem.item = item
        if sysitem.audiofile:
            sysitem.length = (datetime.min + timedelta(seconds=sysitem.audiofile.length)).time()
        sysitem.save()
        calc_total_moderation_time(sysitem.item.moderation)
        return super(ModerationItemSystemCreate, self).form_valid(form)

    def get_success_message(self, cleaned_data):
        return "Moderationsteil %(name)s wurde an Position %(pos)s erstellt." \
            % {'name': self.object.name, 'pos': self.object.item.position}


class ModerationItemSystemUpdate(SuccessMessageMixin, UpdateView):
    model = ModerationItemSystem
    form_class = ModerationItemSystemForm

    success_message = "Moderationsteil %(item.pos) %(name) wurde aktualisisiert"

    def get_context_data(self, **kwargs):
        kwargs['moderation'] = self.object.item.moderation
        context = super(ModerationItemSystemUpdate, self).get_context_data(**kwargs)
        return context

    def get_form(self, *args, **kwargs):
        form = super(ModerationItemSystemUpdate, self).get_form(*args, **kwargs)
        form.fields['audiofile'].choices = [('', '---')] + [
            (a.id, a.report.title + ' / ' + a.title)
            for a in Audiofile.objects.filter(report=self.object.item.moderation.report)]
        return form

    def form_valid(self, form):
        sysitem = form.instance
        if sysitem.audiofile:
            sysitem.length = (datetime.min + timedelta(seconds=sysitem.audiofile.length)).time()
        sysitem.save()
        calc_total_moderation_time(sysitem.item.moderation)
        return super(ModelFormMixin, self).form_valid(form)

    def get_success_message(self, cleaned_data):
        return "Systemteil %(name)s an Position %(pos)s wurde erfolgreich aktualisiert." \
            % {'name': self.object.name, 'pos': self.object.item.position}


class ModerationItemDelete(DeleteView):
    model = ModerationItem

    def delete(self, request, *args, **kwargs):
        obj = self.get_object()
        moderation = obj.moderation
        response = super(ModerationItemDelete, self).delete(request, *args, **kwargs)
        messages.add_message(request, messages.SUCCESS,
                             "Der Moderationsteil wurde erfolgreich gelöscht.")
        calc_total_moderation_time(moderation)
        return response

    def get_success_url(self):
        return self.object.get_absolute_url()


def move_moderation_item(request, pk):

    def top():
        items = ModerationItem.objects.filter(moderation=item.moderation).order_by('position')
        if item.position == 1:
            messages.add_message(request, messages.WARNING,
                                 "Der gewählte Eintrag ist bereits am Anfang der Sequenz")
            return

        for i in items:
            if i.position < item.position:
                i.position += 1
                i.save()
        item.position = 1
        item.save()
        return

    def up():
        if item.position == 1:
            messages.add_message(request, messages.WARNING,
                                 "Der gewählte Eintrag ist bereits am Anfang der Sequenz")
            return

        upper_item = ModerationItem.objects.filter(moderation=item.moderation)\
            .filter(position=item.position-1).get()
        upper_item.position += 1
        upper_item.save()
        item.position -= 1
        item.save()
        return

    def down():
        try:
            lower_item = ModerationItem.objects.filter(moderation=item.moderation)\
                .filter(position=item.position+1).get()
        except ModerationItem.DoesNotExist:
            messages.add_message(request, messages.WARNING,
                                 "Der gewählte Eintrag ist bereits am Ende der Sequenz")
            return

        lower_item.position -= 1
        lower_item.save()
        item.position += 1
        item.save()
        return

    def bottom():
        items = ModerationItem.objects.filter(moderation=item.moderation).order_by('position')

        if item.position == len(items):
            messages.add_message(request, messages.WARNING,
                                 "Der gewählte Eintrag ist bereits am Ende der Sequenz")

        for i in items:
            if i.position > item.position:
                i.position -= 1
                i.save()
        item.position = len(items)
        item.save()
        return

    switcher = {
        'top': top,
        'up': up,
        'down': down,
        'bottom':  bottom
    }

    move = request.GET.get('move')

    item = get_object_or_404(ModerationItem, pk=pk)

    if move not in switcher.keys():
        messages.add_message(request, messages.WARNING,
                             "Der Wert {0} ist nicht gültig für den Parameter move").format(move)
    else:
        func = switcher.get(move, "nothing")
        func()
    return redirect(reverse('show-moderation', kwargs={'pk': item.moderation.pk}))


def calc_total_moderation_time(moderation: Moderation):

    length = timedelta(hours=0, minutes=0, seconds=0, microseconds=0)
    items = ModerationItem.objects.filter(moderation=moderation).select_related('system')
    sys = ModerationItemSystem.objects.filter(item__in=items)

    for i in sys:
        if i.length:
            x = i.length
            s = timedelta(hours=x.hour, minutes=x.minute,
                          seconds=x.second, microseconds=x.microsecond)
            length += s

    moderation.length = (datetime.min + length).time()
    moderation.save()
