import datetime

from django import forms
from flatpickr import DateTimePickerInput

from core.models import Image
from program.models import Broadcast, BroadcastTemplate, WEEKDAYS_BITCHOICE, \
    BroadcastItem, BroadcastTemplateRegularItem, RadioProgramRegularItem, \
    BroadcastRole
from program.widgets import BitChoiceWidget
from moderation.models import Moderation
from django.forms.models import ModelChoiceField
from django.template.loader import render_to_string
from dal import autocomplete
# import martor
# from martor.widgets import MartorWidget

MONTHS = ['Januar', 'Februar', 'März', 'April', 'Mai', 'Juni',
          'Juli', 'August', 'September', 'Oktober', 'November', 'Dezember']

MONTHS_CHOICES = tuple([(i + 1, MONTHS[i]) for i in range(len(MONTHS))])

BROADCAST_VALID_MINUTES = [0, 15, 30, 45]


class BroadcastForm(forms.ModelForm):
    image_file = forms.ImageField(label='Bild', required=False)
    # description = martor.fields.MartorFormField()

    class Meta:
        model = Broadcast
        fields = ('program', 'subtitle', 'start_time', 'end_time', 'description', 'gapfill',
                  'cover_image')

        widgets = {
            # 'description': MartorWidget(attrs={'rows': 2}),
            'start_time': DateTimePickerInput(),
            'end_time': DateTimePickerInput(),
        }
        help_texts = {
            'subtitle': 'Optional. Bitte nur ausfüllen, wenn abweichend/ergänzend von der \
             Sendereihe.',
            'description': 'Optional: beschreibende Information zur Einzelsendung \
             (nicht zur verwendeten Sendungsreihe)',
            'gapfill': 'Sendung weicht in der Anzeige um Zeiten von Sendungen, \
             die keine Lückenfüller sind.'

        }

    def clean_start_time(self):
        data = self.cleaned_data['start_time']
        if (data.hour != 23 and data.minute not in BROADCAST_VALID_MINUTES) or \
           (data.hour == 23 and data.minute not in BROADCAST_VALID_MINUTES + [59]):
                raise forms.ValidationError(
                    'Die Startzeit darf nur Minuten {} enthalten (Ausnahme 23:59).'
                    .format(','.join(str(x) for x in BROADCAST_VALID_MINUTES)))
        return data

    def clean_end_time(self):
        data = self.cleaned_data['end_time']
        if (data.hour != 23 and data.minute not in BROADCAST_VALID_MINUTES) or \
           (data.hour == 23 and data.minute not in BROADCAST_VALID_MINUTES + [59]):
                raise forms.ValidationError(
                    'Die Endzeit darf nur Minuten {} enthalten (Ausnahme 23:59).'
                    .format(','.join(str(x) for x in BROADCAST_VALID_MINUTES)))
        return data

    def save(self, commit=True):
        if self.cleaned_data['image_file']:
            self.instance.cover_image = Image.objects.create(
                    image_file=self.cleaned_data['image_file'])
        return super().save(commit)


class BroadcastEditorForm(forms.ModelForm):
    image_file = forms.ImageField(label='Bild', required=False)
    # description = martor.fields.MartorFormField()

    class Meta:
        model = Broadcast
        fields = ('subtitle', 'description', 'cover_image')

        widgets = {
            # 'description': MartorWidget(attrs={'rows': 2}),
        }
        help_texts = {
            'subtitle': 'Optional. Bitte nur ausfüllen, wenn abweichend/ergänzend von der \
             Sendereihe.',
            'description': 'Optional: beschreibende Information zur Einzelsendung \
             (nicht zur verwendeten Sendungsreihe)',
        }

    def save(self, commit=True):
        if self.cleaned_data['image_file']:
            self.instance.cover_image = Image.objects.create(
                    image_file=self.cleaned_data['image_file'])
        return super().save(commit)


class ModerationForBroadcastItemModelChoiceField(ModelChoiceField):

    def label_from_instance(self, obj):
        return render_to_string('program/broadcast/_broadcastitem_moderation_select.html',
                                {'moderation': obj})


class BroadcastItemForm(forms.ModelForm):

    moderation = ModerationForBroadcastItemModelChoiceField(queryset=None)

    class Meta:
        model = BroadcastItem
        fields = ('time', 'internalnote', 'public', 'moderation')

        widgets = {
            'internalnote': forms.Textarea(attrs={'rows': 3}),
        }
        help_texts = {
            'time': 'Optional',
        }

    def __init__(self, *args, **kwargs):
        super(BroadcastItemForm, self).__init__(*args, **kwargs)

        if self.instance.report is None and self.instance.moderation is not None:
            self.fields['moderation'].queryset = Moderation.objects \
                .filter(pk=self.instance.moderation.pk)
            self.fields['moderation'].empty_label = None

        else:
            self.fields['moderation'].empty_label = "--- Keine Moderation zuordnen ---"
            self.fields['moderation'].required = False
            mod_qs = Moderation.objects.filter(report=self.instance.report)
            self.fields['moderation'].queryset = mod_qs

            # check none type
            if self.instance.moderation is None:
                self.initial['moderation'] = ''

            # preselect single moderation of report on creation
            if self.instance.pk is None and len(mod_qs) == 1:
                self.initial['moderation'] = mod_qs.first()


class BroadcastItemRegularForm(forms.ModelForm):

    class Meta:
        model = BroadcastItem
        fields = ('time', 'internalnote', 'public', 'regular')

        widgets = {
            'internalnote': forms.Textarea(attrs={'rows': 3}),
        }
        help_texts = {
            'time': 'Optional',
        }

    def __init__(self, *args, **kwargs):
        super(BroadcastItemRegularForm, self).__init__(*args, **kwargs)
        self.fields['regular'].queryset = RadioProgramRegularItem.objects\
            .filter(radioprogram=self.instance.broadcast.program)
        self.fields['regular'].empty_label = None


class BroadcastRepeatForm(forms.ModelForm):

    class Meta:
        model = Broadcast
        fields = ('start_time',)

        help_texts = {
            'start_time': 'Format: TT.MM.JJJJ HH:MM:SS',
        }


class BroadcastTemplateForm(forms.ModelForm):

    class Meta:
        model = BroadcastTemplate
        fields = ('program', 'start_time', 'end_time', 'weekdays', 'weekrepeat', 'gapfill')
        help_texts = {
            'weekrepeat': 'Eine Woche entspricht nicht einer Kalenderwoche. \
             Bsp: Ist der 1. des Monats ein Mittwoch, \
             so wird für eine Woche der Zeitraum Mi, 1.X. bis Di, 6.X berücksichtigt.',
        }

    weekdays = forms.IntegerField(label='Wochentage',
                                  widget=BitChoiceWidget(choices=WEEKDAYS_BITCHOICE))


class BroadcastTemplateRegularItemForm(forms.ModelForm):

    class Meta:
        model = BroadcastTemplateRegularItem
        fields = ('regularitem', 'time', 'public')

    def __init__(self, *args, **kwargs):
        super(BroadcastTemplateRegularItemForm, self).__init__(*args, **kwargs)
        self.fields['regularitem'].queryset = RadioProgramRegularItem.\
            objects.filter(radioprogram=self.instance.broadcasttemplate.program)


class BroadcastGenerateRangeForm(forms.Form):

    now = datetime.datetime.now()
    yearchoices = ((now.year, now.year), (now.year + 1, now.year + 1))

    month = forms.IntegerField(widget=forms.Select(choices=MONTHS_CHOICES), initial=now.month + 1)
    year = forms.IntegerField(widget=forms.Select(choices=yearchoices))


class BroadcastDeleteRangeForm(forms.Form):

    from_date = forms.DateField()
    to_date = forms.DateField()

    def clean(self):
        data = super().clean()
        from_d = data.get('from_date')
        to_d = data.get('to_date')

        if from_d and to_d and from_d > to_d:
            raise forms.ValidationError("Enddatum vor Startdatum")

        return data


class BroadcastRoleForm(forms.ModelForm):

    class Meta:
        model = BroadcastRole
        fields = ['name', 'user']
        labels = {
            'user': 'Nutzer*in',
            'name': 'Tätigkeit',
        }
        widgets = {
            'user': autocomplete.ModelSelect2(
                url='user-autocomplete',
                attrs={
                    'data-placeholder': 'Suche nach Nutzer*in',
                    'data-minimum-input-length': 2,
                }
            )
        }
