from django.db import models
from django.urls import reverse

from program.helper import BitChoices
from model_utils.tracker import FieldTracker
from core.models import SoftDeleteModel, django_enum
from enum import Enum
from django.conf import settings


WEEKDAYS_BITCHOICE = BitChoices((('mon', 'Montag'), ('tue', 'Dienstag'), ('wed', 'Mittwoch'),
                                 ('thu', 'Donnerstag'), ('fri', 'Freitag'), ('sat', 'Samstag'),
                                 ('sun', 'Sonntag')))

WEEKDAYS = ((0, 'Montag'), (1, 'Dienstag'), (2, 'Mittwoch'), (3, 'Donnerstag'),
            (4, 'Freitag'), (5, 'Samstag'), (6, 'Sonntag'))

REFERENCE_WEBSITE_TYPES = (('e', 'E-Mail'), ('fb', 'facebook'), ('ig', 'Instagram'),
                           ('mc', 'Mixcloud'), ('sc', 'SoundCloud'), ('sp', 'Spotify'),
                           ('t', 'Telefon'), ('tw', 'Twitter'), ('we', 'Website (extern)'),
                           ('wi', 'Website (intern)'))


class Broadcast(SoftDeleteModel):
    program = models.ForeignKey(
            'RadioProgram', on_delete=models.CASCADE, verbose_name='Sendereihe')
    subtitle = models.CharField('Untertitel', max_length=255, blank=True)
    start_time = models.DateTimeField('Beginn der Sendung')
    end_time = models.DateTimeField('Ende der Sendung')
    description = models.TextField('Beschreibung', blank=True, null=True)
    firstrun = models.ForeignKey("self", blank=True, null=True,
                                 related_name='repeats', on_delete=models.CASCADE)
    reports = models.ManyToManyField('audiothek.RadioReport', related_name='broadcasts')
    gapfill = models.BooleanField('Lückenfüller', blank=True, default=False)
    cover_image = models.ForeignKey('core.Image', on_delete=models.SET_NULL,
                                    null=True, blank=True)

    tracker = FieldTracker()

    class Meta:
        ordering = ['start_time']

    def __str__(self):
        return '{}'.format(self.program) + \
            (' : {}'.format(self.subtitle) if self.subtitle else '')

    def get_absolute_url(self):
        return reverse('show-broadcast', args=(self.pk,))


class BroadcastRole(models.Model):

    @django_enum
    class Name(Enum):
        host = "Moderation"
        news = "Nachrichten"
        editor = "Redaktion"
        chief = "Redaktion vom Dienst (RvD)"

        def __repr__(self):
            return self.name

        def __str__(self):
            return self.value

    name = models.CharField(max_length=31, choices=[(role.name, role.value) for role in Name])
    user = models.ForeignKey(settings.AUTH_USER_MODEL, on_delete=models.PROTECT)

    broadcast = models.ForeignKey('program.Broadcast', on_delete=models.CASCADE,
                                  null=True, blank=True, related_name='roles')

    def get_name(self):
        return self.Name[self.name]


class RadioProgram(models.Model):
    name = models.CharField(max_length=100)
    description = models.TextField(max_length=3000, blank=True)
    teaser = models.CharField(max_length=180, blank=True)
    cover_image = models.ForeignKey('core.Image', on_delete=models.SET_NULL,
                                    null=True, blank=True)

    class Meta:
        ordering = ('name',)

    def __str__(self):
        return self.name

    def get_absolute_url(self):
        return reverse('show-radio-program', args=(self.pk,))


class RadioProgramReference(models.Model):
    radio_program = models.ForeignKey(
        'RadioProgram', on_delete=models.CASCADE,
        verbose_name='Sendereihe', related_name='references')
    type = models.CharField(max_length=255, choices=REFERENCE_WEBSITE_TYPES)
    url = models.CharField(max_length=255)
    display = models.CharField(max_length=255, blank=True)

    def __str__(self):
        return '{}'.format(self.radio_program) + \
            ('  : {} – {} ({})'.format(self.type, self.url, self.display))


class RadioProgramRegularItem(models.Model):
    title = models.CharField('Titel', max_length=255)
    description = models.TextField(max_length=3000, blank=True)
    radioprogram = models.ForeignKey(RadioProgram, on_delete=models.PROTECT,
                                     related_name='regularitems', null=True)

    def __str__(self):
        return '{} ({})'.format(self.title, self.radioprogram)


class BroadcastItem(models.Model):
    broadcast = models.ForeignKey(Broadcast, on_delete=models.CASCADE, related_name='items')
    position = models.PositiveSmallIntegerField('Position', default=1)
    time = models.TimeField('Sendezeit', blank=True, null=True)
    internalnote = models.TextField('Interne Notiz', blank=True)
    public = models.BooleanField('Öffentlich sichtbar', default=True)

    regular = models.ForeignKey(RadioProgramRegularItem, on_delete=models.SET_NULL,
                                related_name='broadcastitems', null=True)
    report = models.ForeignKey('audiothek.RadioReport', on_delete=models.SET_NULL,
                               related_name='broadcastitems', null=True)
    moderation = models.ForeignKey('moderation.Moderation', on_delete=models.SET_NULL, null=True,
                                   related_name='broadcastitems')

    tracker = FieldTracker()

    def save(self, *args, **kwargs):
        if not self.pk:
            existingItems = BroadcastItem.objects.filter(broadcast=self.broadcast)
            self.position = existingItems.count()+1
        super(BroadcastItem, self).save()

    def delete(self):
        existHigherPos = BroadcastItem.objects.filter(
            broadcast=self.broadcast).filter(position__gt=self.position)
        if existHigherPos:
            for p in existHigherPos:
                p.position -= 1
                p.save()
        super(BroadcastItem, self).delete()

    class Meta:
        ordering = ['broadcast__start_time', 'position']

    def get_absolute_url(self):
        return reverse('show-broadcast', args=[self.broadcast.pk])

    def __str__(self):
        if self.regular:
            return self.regular.title
        elif self.report:
            return self.report.title
        else:
            return "?"


class BroadcastTemplate(models.Model):

    WEEKLY = 'W'
    BIWEEKLY_EVEN = 'E'
    BIWEEKLY_UNEVEN = 'U'
    MONTH_WEEK_1 = '1'
    MONTH_WEEK_2 = '2'
    MONTH_WEEK_3 = '3'
    MONTH_WEEK_4 = '4'
    MONTH_WEEK_LAST = 'L'

    WEEK_ROUTINE_CHOICES = (
        (WEEKLY, 'wöchentlich'),
        (BIWEEKLY_EVEN, '14-tägig (gerade Kalenderwoche)'),
        (BIWEEKLY_EVEN, '14-tägig (ungerade Kalenderwoche)'),
        (MONTH_WEEK_1, '1. Woche im Monat'),
        (MONTH_WEEK_2, '2. Woche im Monat'),
        (MONTH_WEEK_3, '3. Woche im Monat'),
        (MONTH_WEEK_4, '4. Woche im Monat'),
        (MONTH_WEEK_LAST, 'Letzte Woche im Monat'),
        )

    program = models.ForeignKey(
            'RadioProgram',
            on_delete=models.CASCADE,
            verbose_name='Sendereihe')
    weekdays = models.PositiveIntegerField('Wochentage', default=0)

    start_time = models.TimeField('Startzeit')
    end_time = models.TimeField('Endzeit')
    weekrepeat = models.CharField('Wochenturnus',
                                  max_length=1,
                                  choices=WEEK_ROUTINE_CHOICES,
                                  default=WEEKLY)
    gapfill = models.BooleanField('Lückenfüller für ungeplante Zeiten',
                                  default=False)

    def __str__(self):
        return self.program.name

    def get_absolute_url(self):
        return reverse('show-broadcast-template', args=[self.pk])

    class Meta:
        ordering = ('start_time',)


class BroadcastTemplateRegularItem(models.Model):
    broadcasttemplate = models.ForeignKey(
        'BroadcastTemplate',
        on_delete=models.CASCADE,
        related_name='broadcasttemplateregularitems',
        verbose_name='Sendungsvorlage')
    regularitem = models.ForeignKey(
        'RadioProgramRegularItem',
        related_name='broadcasttemplateregularitems',
        on_delete=models.CASCADE,
        verbose_name='Stamm-Sendeinhalt')
    position = models.PositiveSmallIntegerField('Position', default=1)
    time = models.TimeField('Sendezeit', blank=True, null=True)
    public = models.BooleanField('Öffentlich sichtbar', default=True)

    def save(self, *args, **kwargs):
        if not self.pk:
            existingItems = BroadcastTemplateRegularItem.objects\
                .filter(broadcasttemplate=self.broadcasttemplate)
            self.position = existingItems.count()+1
        super(BroadcastTemplateRegularItem, self).save()

    def delete(self):
        existHigherPos = BroadcastTemplateRegularItem.objects.filter(
            broadcasttemplate=self.broadcasttemplate).filter(position__gt=self.position)
        if existHigherPos:
            for p in existHigherPos:
                p.position -= 1
                p.save()
        super(BroadcastTemplateRegularItem, self).delete()

    def get_absolute_url(self):
        return reverse('show-broadcast-template', args=[self.broadcasttemplate.pk])

    def __str__(self):
        return '{} - {}'.format(self.broadcasttemplate, self.regularitem)

    class Meta:
        ordering = ('position',)


class BroadcastRepeatTemplate(models.Model):
    origin = models.ForeignKey(
            'BroadcastTemplate',
            on_delete=models.CASCADE,
            related_name='broadcastrepeattemplates',
            verbose_name='Vorlage der Erstsendung')
    plusweek = models.PositiveSmallIntegerField(default=0,
                                                verbose_name='Wochen nach Erstaustrahlung')
    weekday = models.PositiveIntegerField('Wochentag', default=0, choices=WEEKDAYS)
    start_time = models.TimeField('Startzeit')

    def __str__(self):
        return '{} +{} {}'.format(self.weekday, self.plusweek, self.start_time)

    def get_absolute_url(self):
        return reverse('show-broadcast-template', args=[self.origin.pk])
