from core.util import get_referer_view
from datetime import time
from django.contrib import messages
from django.contrib.messages.views import SuccessMessageMixin
from django.shortcuts import get_object_or_404, redirect
from django.urls.base import reverse, resolve
from django.views.generic.edit import CreateView, UpdateView, DeleteView
from django_filters.views import FilterView

from audiothek.filters import RadioReportFilter
from audiothek.models import RadioReport
from moderation.filters import ModerationFilter
from moderation.models import Moderation
from program.models import Broadcast, BroadcastItem, RadioProgramRegularItem
from program.forms import BroadcastItemForm, BroadcastItemRegularForm
from actstream.signals import action
from activity import verbs
from django.core.exceptions import ObjectDoesNotExist


BROADCAST_ITEM_TRACKFIELDS = ['regular_id', 'report_id', 'moderation_id', 'time', 'public']

# BroadcastItem, applicable for all variations


class BroadcastItemUpdate(UpdateView, SuccessMessageMixin):
    model = BroadcastItem
    form_class = BroadcastItemForm
    template_name = 'program/broadcast/broadcastitem_form.html'

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        bcitem = get_object_or_404(BroadcastItem, pk=self.kwargs.get('pk'))
        kwargs['instance'] = bcitem
        return kwargs

    def get_context_data(self, **kwargs):
        context = super(BroadcastItemUpdate, self).get_context_data(**kwargs)
        context['broadcast'] = self.object.broadcast

        if self.object.report:
            context['report'] = self.object.report
        if self.object.moderation:
            context['moderation'] = self.object.moderation

        referer_view = get_referer_view(self.request)
        if referer_view:
            context['referer_view'] = resolve(referer_view)
            context['referer_url'] = referer_view

        return context

    def form_valid(self, form):
        self.object = form.save(commit=False)
        changes = build_broadcastitem_changes(self.object, BROADCAST_ITEM_TRACKFIELDS)

        if changes:
            action.send(self.request.user,
                        verb=verbs.CHANGED_BROADCASTITEM,
                        action_object=self.object,
                        target=self.object.broadcast,
                        changes=changes)

        messages.add_message(self.request, messages.SUCCESS,
                             "Der Beitrag  %(report)s wurde der Sendung %(broadcast)s angefügt."
                             % {'report': self.object.report,
                                'broadcast': self.object.broadcast})

        return super(BroadcastItemUpdate, self).form_valid(form)

    def get_success_url(self):
        return self.request.POST.get('referer_url', self.object.broadcast.get_absolute_url())


class BroadcastItemDelete(DeleteView):
    model = BroadcastItem
    template_name = 'program/broadcast/broadcastitem_confirm_delete.html'

    def get_context_data(self, **kwargs):
        context = super(BroadcastItemDelete, self).get_context_data(**kwargs)
        referer_view = get_referer_view(self.request)
        if referer_view:
            context['referer_view'] = resolve(referer_view)
            context['referer_url'] = referer_view
        return context

    def delete(self, request, *args, **kwargs):
        response = super(BroadcastItemDelete, self).delete(request, *args, **kwargs)
        messages.add_message(request, messages.SUCCESS, "Sendungsinhalt erfolgreich gelöscht.")
        return response

    def get_success_url(self):
        return self.request.POST.get('referer_url', self.object.get_absolute_url())


def move_broadcast_item(request, bcitem_pk, action):

    def top():
        items = BroadcastItem.objects.filter(broadcast=bcitem.broadcast).order_by('position')
        if bcitem.position == 1:
            messages.add_message(request, messages.WARNING,
                                 "Der gewählte Eintrag ist bereits am Anfang der Liste")
            return

        for i in items:
            if i.position < bcitem.position:
                i.position += 1
                i.save()
        bcitem.position = 1
        bcitem.save()
        return

    def up():
        if bcitem.position == 1:
            messages.add_message(request, messages.WARNING,
                                 "Der gewählte Eintrag ist bereits am Anfang der Liste")
            return

        upper_item = BroadcastItem.objects.filter(broadcast=bcitem.broadcast)\
            .filter(position=bcitem.position-1).get()
        upper_item.position += 1
        upper_item.save()
        bcitem.position -= 1
        bcitem.save()
        return

    def down():
        try:
            lower_item = BroadcastItem.objects.filter(broadcast=bcitem.broadcast)\
                .filter(position=bcitem.position+1).get()
        except BroadcastItem.DoesNotExist:
            messages.add_message(request, messages.WARNING,
                                 "Der gewählte Eintrag ist bereits am Ende der Liste")
            return

        lower_item.position -= 1
        lower_item.save()
        bcitem.position += 1
        bcitem.save()
        return

    def bottom():
        items = BroadcastItem.objects.filter(broadcast=bcitem.broadcast).order_by('position')

        if bcitem.position == len(items):
            messages.add_message(request, messages.WARNING,
                                 "Der gewählte Eintrag ist bereits am Ende der Sequenz")

        for i in items:
            if i.position > bcitem.position:
                i.position -= 1
                i.save()
        bcitem.position = len(items)
        bcitem.save()
        return

    switcher = {
        'top': top,
        'up': up,
        'down': down,
        'bottom':  bottom
    }

    bcitem = get_object_or_404(BroadcastItem, pk=bcitem_pk)

    if action not in switcher.keys():
        messages.add_message(request, messages.WARNING,
                             "Der Wert {0} ist nicht gültig für den Parameter move").format(action)
    else:
        func = switcher.get(action, "nothing")
        func()
    return redirect(
        reverse('show-broadcast', kwargs={'pk': bcitem.broadcast.pk}) + '#broadcastitems')


def order_items_of_broadcast_by_time(request, pk):

    bc = get_object_or_404(Broadcast, pk=pk)
    bcitems = BroadcastItem.objects.filter(broadcast=bc)

    if bcitems is None:
        messages.add_message(request, messages.INFO,
                             "Keine Sendungsinhalte zum Sortieren")
        return redirect(
            reverse('show-broadcast', kwargs={'pk': bc.pk}) + '#broadcastitems')

    sorted_list = []

    for item in bcitems:

        check_time = item.time or time.max
        inserted = False

        for i in range(len(sorted_list)):

            sorted_time = sorted_list[i].time or time.max

            if sorted_time > check_time:
                sorted_list.insert(i, item)
                inserted = True
                break

        if not inserted:
            sorted_list.append(item)

    for i in range(len(sorted_list)):
        sitem = sorted_list[i]
        sitem.position = i+1
        sitem.save()

    return redirect(
        reverse('show-broadcast', kwargs={'pk': bc.pk}) + '#broadcastitems')


# BroadcastItem-RadioReport relation

class BroadcastRadioReportSelect(FilterView):
    filterset_class = RadioReportFilter
    context_object_name = 'reports'
    paginate_by = 10
    template_name = 'program/broadcast/broadcast_select_report.html'

    def get_context_data(self, **kwargs):
        context = super(BroadcastRadioReportSelect, self).get_context_data(**kwargs)
        context['broadcast'] = get_object_or_404(Broadcast, pk=self.kwargs.get('broadcast_pk'))
        return context


class BroadcastItemReportCreate(CreateView, SuccessMessageMixin):
    model = BroadcastItem
    form_class = BroadcastItemForm
    template_name = 'program/broadcast/broadcastitem_form.html'

    def get_context_data(self, **kwargs):
        context = super(BroadcastItemReportCreate, self).get_context_data(**kwargs)
        context['broadcast'] = get_object_or_404(Broadcast, pk=self.kwargs.get('broadcast_pk'))
        return context

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        broadcast = get_object_or_404(Broadcast, pk=self.kwargs.get('broadcast_pk'))
        report = get_object_or_404(RadioReport, pk=self.kwargs.get('report_pk'))

        if report.moderations.count() > 1:
            messages.add_message(
                self.request,
                messages.INFO,
                "Zum Beitrag '%(b)s' existieren aktuell mehrerer Moderationen. \
                Initial ist daher keine Moderation für den Sendungsinhalt ausgewählt. \
                Bitte wähle eine, falls gewünscht." % {'b': report}
            )

        kwargs['instance'] = BroadcastItem(report=report, broadcast=broadcast)
        return kwargs

    def form_valid(self, form):
        self.object = form.save(commit=False)

        changes = build_broadcastitem_changes(self.object, BROADCAST_ITEM_TRACKFIELDS)
        action.send(self.request.user,
                    verb=verbs.ADDED_BROADCASTITEM,
                    action_object=self.object,
                    target=self.object.broadcast,
                    changes=changes)

        messages.add_message(self.request, messages.SUCCESS,
                             "Der Beitrag  %(report)s wurde der Sendung %(broadcast)s angefügt."
                             % {'report': self.object.report,
                                'broadcast': self.object.broadcast})

        return super(BroadcastItemReportCreate, self).form_valid(form)

    def get_success_url(self):
        return self.object.broadcast.get_absolute_url()


def quick_add_report_to_broadcast(request, broadcast_pk, report_pk):

    broadcast = get_object_or_404(Broadcast, pk=broadcast_pk)
    report = get_object_or_404(RadioReport, pk=report_pk)

    bcitem = BroadcastItem()
    bcitem.broadcast = broadcast
    bcitem.report = report

    if report.moderations.count() == 1:
        bcitem.moderation = report.moderations.first()

    changes = build_broadcastitem_changes(bcitem, BROADCAST_ITEM_TRACKFIELDS)
    action.send(request.user,
                verb=verbs.ADDED_BROADCASTITEM,
                action_object=bcitem,
                target=bcitem.broadcast,
                changes=changes)

    bcitem.save()

    messages.add_message(request, messages.SUCCESS,
                         "Der Beitrag '%(r)s' wurde der Sendung %(b)s hinzugefügt." %
                         {'r': report, 'b': broadcast})

    if report.moderations.count() == 1:
        messages.add_message(
            request,
            messages.INFO,
            "Zum Beitrag '%(b)s' existiert aktuell eine Moderation '%(m)s'. \
            Diese wurde dem Sendeinhalt mit zugeordnet"
            % {'b': bcitem.report, 'm': bcitem.moderation})

    elif report.moderations.count() > 1:
        messages.add_message(
            request,
            messages.INFO,
            "Zum Beitrag '%(b)s' existieren mehrere Moderationen. \
            Es wurde dem Sendeinhalt keine zugordnet. \
            Bitte wähle manuell eine aus." % {'b': bcitem.report})

    return redirect(reverse('show-broadcast', kwargs={'pk': broadcast.pk}))


# BroadcastItem-Moderation relation

class BroadcastModerationSelect(FilterView):
    filterset_class = ModerationFilter
    context_object_name = 'moderations'
    template_name = 'program/broadcast/broadcast_select_moderation.html'
    paginate_by = 10

    def get_context_data(self, **kwargs):
        context = super(BroadcastModerationSelect, self).get_context_data(**kwargs)
        context['broadcast'] = get_object_or_404(Broadcast, pk=self.kwargs.get('broadcast_pk'))
        return context


class BroadcastItemModerationCreate(CreateView, SuccessMessageMixin):
    model = BroadcastItem
    form_class = BroadcastItemForm
    template_name = 'program/broadcast/broadcastitem_form.html'

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        broadcast = get_object_or_404(Broadcast, pk=self.kwargs.get('broadcast_pk'))
        moderation = get_object_or_404(Moderation, pk=self.kwargs.get('moderation_pk'))
        kwargs['instance'] = BroadcastItem(moderation=moderation,
                                           broadcast=broadcast, report=moderation.report)
        return kwargs

    def get_context_data(self, **kwargs):
        context = super(BroadcastItemModerationCreate, self).get_context_data(**kwargs)
        context['broadcast'] = get_object_or_404(Broadcast, pk=self.kwargs.get('broadcast_pk'))
        return context

    def form_valid(self, form):
        self.object = form.save(commit=False)
        broadcast = get_object_or_404(Broadcast, pk=self.kwargs.get('broadcast_pk'))
        moderation = get_object_or_404(Moderation, pk=self.kwargs.get('moderation_pk'))

        self.object.broadcast = broadcast
        self.object.moderation = moderation

        if moderation.report:
            self.object.report = moderation.report

        changes = build_broadcastitem_changes(self.object, BROADCAST_ITEM_TRACKFIELDS)

        if changes:
            action.send(self.request.user,
                        verb=verbs.ADDED_BROADCASTITEM,
                        action_object=self.object,
                        target=self.object.broadcast,
                        changes=changes)

        messages.add_message(self.request, messages.SUCCESS,
                             "Die Moderation  %(mod)s wurde der Sendung %(broadcast)s angefügt."
                             % {'mod': self.object.moderation,
                                'broadcast': self.object.broadcast})

        return super(BroadcastItemModerationCreate, self).form_valid(form)

    def get_success_url(self):
        return self.object.broadcast.get_absolute_url()


def quick_add_moderation_to_broadcast(request, broadcast_pk, moderation_pk):

    broadcast = get_object_or_404(Broadcast, pk=broadcast_pk)
    moderation = get_object_or_404(Moderation, pk=moderation_pk)

    bcitem = BroadcastItem()
    bcitem.broadcast = broadcast
    bcitem.moderation = moderation

    if moderation.report:
        bcitem.report = moderation.report

    changes = build_broadcastitem_changes(bcitem, BROADCAST_ITEM_TRACKFIELDS)
    action.send(request.user,
                verb=verbs.ADDED_BROADCASTITEM,
                action_object=bcitem,
                target=bcitem.broadcast,
                changes=changes)

    bcitem.save()

    messages.add_message(request, messages.SUCCESS,
                         "Die Moderation %(m)s wurde der Sendung %(b)s hinzugefügt." %
                         {'m': moderation, 'b': broadcast})

    if moderation.report:
        messages.add_message(
            request, messages.INFO,
            "Zur Moderation '%(m)s' ist akutell der Beitrag '%(b)s' zugeordnet. \
            Dieser wurde auch dem Sendeinhalt mit zugeordnet"
            % {'b': bcitem.report, 'm': bcitem.moderation})

    return redirect(reverse('show-broadcast', kwargs={'pk': broadcast.pk}))


# BroadcastItem-RegularItem relation

class BroadcastItemRegularCreate(CreateView, SuccessMessageMixin):
    model = BroadcastItem
    form_class = BroadcastItemRegularForm
    template_name = 'program/broadcast/broadcastitem_regularitem_form.html'

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        b = get_object_or_404(Broadcast, pk=self.kwargs.get('broadcast_pk'))
        kwargs['instance'] = BroadcastItem(broadcast=b)
        return kwargs

    def get_context_data(self, **kwargs):
        context = super(BroadcastItemRegularCreate, self).get_context_data(**kwargs)
        b = get_object_or_404(Broadcast, pk=self.kwargs.get('broadcast_pk'))
        context['broadcast'] = b
        return context

    def form_valid(self, form):
        self.object = form.save(commit=False)

        changes = build_broadcastitem_changes(self.object, BROADCAST_ITEM_TRACKFIELDS)

        if changes:
            action.send(self.request.user,
                        verb=verbs.ADDED_BROADCASTITEM,
                        action_object=self.object,
                        target=self.object.broadcast,
                        changes=changes)

        messages.add_message(self.request, messages.SUCCESS,
                             "Der Stammsendeinhalt  %(regular)s wurde \
                             der Sendung %(broadcast)s angefügt."
                             % {'regular': self.object.regular,
                                'broadcast': self.object.broadcast})

        return super(BroadcastItemRegularCreate, self).form_valid(form)

    def get_success_url(self):
        return self.object.broadcast.get_absolute_url()


class BroadcastItemRegularUpdate(UpdateView, SuccessMessageMixin):
    model = BroadcastItem
    form_class = BroadcastItemRegularForm
    template_name = 'program/broadcast/broadcastitem_regularitem_form.html'

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        bcitem = get_object_or_404(BroadcastItem, pk=self.kwargs.get('pk'))
        kwargs['instance'] = bcitem
        return kwargs

    def get_context_data(self, **kwargs):
        context = super(BroadcastItemRegularUpdate, self).get_context_data(**kwargs)
        context['broadcast'] = self.object.broadcast
        return context

    def form_valid(self, form):
        self.object = form.save(commit=False)
        changes = build_broadcastitem_changes(self.object, BROADCAST_ITEM_TRACKFIELDS)

        if changes:
            action.send(self.request.user,
                        verb=verbs.CHANGED_BROADCASTITEM,
                        action_object=self.object,
                        target=self.object.broadcast,
                        changes=changes)

        messages.add_message(self.request, messages.SUCCESS,
                             "Der Stammsendeinhalt  %(regular)s \
                             wurde der Sendung %(broadcast)s angefügt."
                             % {'regular': self.object.regular,
                                'broadcast': self.object.broadcast})

        return super(BroadcastItemRegularUpdate, self).form_valid(form)

    def get_success_url(self):
        return self.object.broadcast.get_absolute_url()


def build_broadcastitem_changes(broadcast, trackfields):

    changes = {}

    for trackfield in trackfields:

        if trackfield in broadcast.tracker.changed().keys():

            changes[trackfield] = getattr(broadcast, trackfield)

            if not changes[trackfield] and broadcast.tracker.previous(trackfield) is None:
                del changes[trackfield]
                continue

            if trackfield == 'time':
                changes[trackfield] = changes[trackfield].strftime("%H:%M:%S")

            if trackfield == 'regular_id':
                try:
                    r = RadioProgramRegularItem.objects.get(pk=changes[trackfield])
                    changes[trackfield] = "{} ({})".format(r.pk, r.title)
                except ObjectDoesNotExist:
                    pass

            if trackfield == 'report_id':
                try:
                    r = RadioReport.objects.get(pk=changes[trackfield])
                    changes[trackfield] = "{} ({})".format(r.pk, r.title)
                except ObjectDoesNotExist:
                    pass

            if trackfield == 'moderation_id':
                try:
                    m = Moderation.objects.get(pk=changes[trackfield])
                    changes[trackfield] = "{} ({})".format(m.pk, m.name)
                except ObjectDoesNotExist:
                    changes[trackfield] = ""

    return changes
