from builtins import int
from core.util import get_referer_view, parse_date
from datetime import timedelta, datetime, date, time
from django.contrib import messages
from django.contrib.messages.views import SuccessMessageMixin
from django.forms.widgets import HiddenInput
from django.http.response import HttpResponseRedirect, HttpResponseForbidden
from django.shortcuts import get_object_or_404, render, redirect
from django.urls import reverse_lazy, reverse
from django.utils.http import urlencode
from django.views.generic import CreateView, DetailView, UpdateView, DeleteView
from django_filters.views import FilterView
import pytz

from program.filters import BroadcastFilter
from program.forms import BroadcastGenerateRangeForm, MONTHS_CHOICES, BroadcastForm,\
    BroadcastRepeatForm, BroadcastDeleteRangeForm, BroadcastRoleForm,\
    BroadcastEditorForm
from program.generateprogram import query_broadcast_templates_of_day,\
    last_day_of_month, broadcast_from_template, repeat_broadcast_from_template,\
    create_broadcast_regularitems_from_template
from program.models import Broadcast, RadioProgram, BroadcastRole
from lohrothek import settings
from actstream.signals import action
from activity import verbs
from django.views.generic.edit import FormMixin
from django.views.generic.base import View
from django.utils.decorators import method_decorator
from django.contrib.admin.views.decorators import staff_member_required
from program.checkprogram import check_program_irregularity_day,\
    BroadcastIrregularity
from django.utils.timezone import localtime


BROADCAST_TRACKFIELDS = ['description', 'subtitle', 'start_time', 'end_time', 'program_id']
BROADCAST_REPEAT_TRACKFIELDS = ['start_time', 'end_time']


class BroadcastList(FilterView):
    filterset_class = BroadcastFilter
    context_object_name = 'broadcasts'
    template_name = 'program/broadcast/broadcast_list.html'
    strict = False

    def dispatch(self, request, *args, **kwargs):

        SESSION_BROADCAST_LIST_DAY_KEY = 'list-broadcasts-day'
        date_str = self.request.GET.get('date')

        # no get param provided
        if date_str is None:
            # check session for previous or curren date
            date_str = request.session.get(SESSION_BROADCAST_LIST_DAY_KEY,
                                           datetime.now().strftime("%d.%m.%Y"))
            return redirect('{}?{}'.format(
                reverse('list-broadcasts'),
                urlencode({'date': date_str})
                ))
        # get param provided
        else:
            date = parse_date(date_str)
            # invalid date
            if date is None:
                messages.add_message(self.request, messages.WARNING,
                                     "Das angegebene Datum %(d)s ist ungültig. \
                                     Umleitung zum heutigen Datum" % {'d': date_str})
                return redirect('{}?{}'.format(
                    reverse('list-broadcasts'),
                    urlencode({'date': datetime.now().strftime("%d.%m.%Y")})
                ))
            else:
                request.session[SESSION_BROADCAST_LIST_DAY_KEY] = date_str

        return super().dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):

        context = super().get_context_data(**kwargs)
        date_str = self.request.GET.get('date')

        if date_str is None:
            qdate = datetime.now()
        else:
            qdate = datetime.strptime(date_str, "%d.%m.%Y").date()

        start = qdate - timedelta(days=qdate.weekday())

        context['date'] = qdate
        context['prevsunday'] = start - timedelta(days=1)
        context['prevdate'] = qdate - timedelta(days=1)
        context['nextdate'] = qdate + timedelta(days=1)

        weekdates = [start]
        for i in range(1, 7):
            weekdates.append(start + timedelta(days=i))
        context['weekdates'] = weekdates

        context['nextmonday'] = weekdates[len(weekdates)-1] + timedelta(days=1)

        for i in check_program_irregularity_day(context['object_list']):

            if i[0] == BroadcastIrregularity.overlap:
                messages.add_message(
                    self.request,
                    messages.ERROR,
                    ("Die folgenden Sendungen überschneiden sich: " +
                     "%(one)s (%(onestart)s-%(oneend)s) -" +
                     " %(two)s (%(twostart)s-%(twoend)s)") %
                    {'one': i[1].program,
                     'onestart': localtime(i[1].start_time).strftime("%H:%M"),
                     'oneend': localtime(i[1].end_time).strftime("%H:%M"),
                     'two': i[2].program,
                     'twostart': localtime(i[2].start_time).strftime("%H:%M"),
                     'twoend': localtime(i[2].end_time).strftime("%H:%M")
                     })

            elif i[0] == BroadcastIrregularity.empty_between:
                messages.add_message(
                    self.request,
                    messages.WARNING,
                    ("Zwischen den  folgenden Sendungen ist kein Programm: " +
                     "%(one)s Ende %(oneend)s <> " +
                     "%(twostart)s Beginn %(two)s") %
                    {'one': i[1].program,
                     'oneend': localtime(i[1].end_time).strftime("%H:%M"),
                     'two': i[2].program,
                     'twostart': localtime(i[2].start_time).strftime("%H:%M"),
                     })

            elif i[0] == BroadcastIrregularity.empty_before:
                messages.add_message(
                    self.request,
                    messages.WARNING,
                    ("Vor der folgenden Sendungen ist kein Programm: " +
                     "%(one)s (%(onestart)s - %(oneend)s)") %
                    {'one': i[1].program,
                     'onestart': localtime(i[1].start_time).strftime("%H:%M"),
                     'oneend': localtime(i[1].end_time).strftime("%H:%M"),
                     })

            elif i[0] == BroadcastIrregularity.empty_after:
                messages.add_message(
                    self.request,
                    messages.WARNING,
                    ("Nach der folgenden Sendungen ist kein Programm: " +
                     "%(one)s (%(onestart)s - %(oneend)s)") %
                    {'one': i[1].program,
                     'onestart': localtime(i[1].start_time).strftime("%H:%M"),
                     'oneend': localtime(i[1].end_time).strftime("%H:%M"),
                     })

        return context


class BroadcastDetail(View):
    def get(self, request, *args, **kwargs):
        view = BroadcastDisplay.as_view()
        return view(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        view = BroadcastRoleCreate.as_view()
        return view(request, *args, **kwargs)


class BroadcastDisplay(DetailView):
    model = Broadcast
    template_name = 'program/broadcast/broadcast_detail.html'

    def get_context_data(self, **kwargs):
        context = super(BroadcastDisplay, self).get_context_data(**kwargs)

        initial = {
            'user': self.request.user,
            'name': BroadcastRole.Name.host.name,
            }

        context['formroles'] = BroadcastRoleForm(initial=initial)
        return context


class BroadcastRoleCreate(CreateView, FormMixin):
    model = BroadcastRole
    form_class = BroadcastRoleForm
    template_name = 'program/broadcast/broadcast_detail.html'

    def form_valid(self, form):
        bpk = self.kwargs.get('pk')
        broadcast = get_object_or_404(Broadcast, pk=bpk)

        self.object = form.save(commit=False)
        self.object.broadcast = broadcast

        existingRoles = BroadcastRole.objects.filter(broadcast=broadcast)

        for ex in existingRoles:
            if ex.user == self.object.user and ex.name == self.object.name:
                messages.add_message(self.request, messages.ERROR,
                                     "Nutzer*in [%(u)s] wurde für die Sendung bereits \
                                     die Rolle [%(r)s] zugewiesen." %
                                     {'u': self.object.user,
                                      'r': BroadcastRole.Name[self.object.name]})
                return redirect(self.get_success_url())

        messages.add_message(self.request, messages.SUCCESS,
                             "Nutzer*in [%(u)s] wurde mit Rolle [%(r)s] für die Sendung erstelt."
                             % {'u': self.object.user, 'r': BroadcastRole.Name[self.object.name]})

        return CreateView.form_valid(self, form)

    def get_success_url(self):
        return reverse('show-broadcast', kwargs={'pk': self.object.broadcast.pk})


class BroadcastRoleDelete(DeleteView):
    model = BroadcastRole
    template_name = 'program/broadcast/broadcastrole_confirm_delete.html'

    def get_success_url(self):
        return reverse('show-broadcast', kwargs={'pk': self.object.broadcast.pk})


@method_decorator(staff_member_required, name='dispatch')
class BroadcastCreate(CreateView):
    model = Broadcast
    form_class = BroadcastForm
    template_name = 'program/broadcast/broadcast_form.html'

    def form_valid(self, form):

        changes = build_broadcast_changes(form.instance, BROADCAST_TRACKFIELDS)
        self.object = form.save()

        if changes:
            action.send(self.request.user, verb=verbs.CREATED_BROADCAST,
                        action_object=self.object,
                        changes=changes)

        return CreateView.form_valid(self, form)


@staff_member_required
def createBroadcastsFromTemplate(request):
    if not request.user.is_staff:
        raise HttpResponseForbidden

    context = {}
    context['rangeform'] = BroadcastGenerateRangeForm()

    if ('month' not in request.GET.keys()) and ('year' not in request.GET.keys()):
        return render(request, 'program/broadcast/broadcast_create_from_templates.html', context)

    query_month = request.GET.get('month')
    query_year = request.GET.get('year')
    valid_params = True

    if not query_year.isnumeric():
        messages.warning(request, "year parameter is not integer")
        valid_params = False
    if not query_month.isnumeric():
        messages.warning(request, "month parameter is not integer")
        valid_params = False

    if int(query_month) < 1 or int(query_month) > 12:
        messages.warning(request, "month parameter not 1...12")
        valid_params = False

    if valid_params:

        broadcasts = []

        for d in range(1, last_day_of_month(date(int(query_year), int(query_month), 1))):

            querydate = date(int(query_year), int(query_month), d)

            for t in query_broadcast_templates_of_day(querydate):

                bc = broadcast_from_template(t, querydate)
                if request.method == 'POST':
                    bc.save()
                    create_broadcast_regularitems_from_template(t, bc)
                broadcasts.append(bc)

                for r in t.broadcastrepeattemplates.all():
                    rbc = repeat_broadcast_from_template(r, bc)
                    if request.method == 'POST':
                        rbc.save()
                    broadcasts.append(rbc)

        broadcasts.sort(key=lambda x: x.start_time)

        if request.method == 'POST':
            messages.info(request, "Sendungen für Monat {} {} wurden erstellt."
                          .format(dict(MONTHS_CHOICES)[int(query_month)], query_year))
            return HttpResponseRedirect(reverse('create-broadcasts-from-templates'))

        context['broadcasts'] = broadcasts

    return render(request, 'program/broadcast/broadcast_create_from_templates.html', context)


class BroadcastUpdate(UpdateView):
    model = Broadcast
    template_name = 'program/broadcast/broadcast_form.html'

    def get_context_data(self, **kwargs):
        context = super(BroadcastUpdate, self).get_context_data(**kwargs)
        referer_view = get_referer_view(self.request, None, True)
        if referer_view:
            context['referer_url'] = referer_view
        else:
            context['referer_url'] = 'show-broadcast'
        return context

    def get_form_class(self):
        if self.request.user.is_staff:
            self.form_class = BroadcastForm
        else:
            self.form_class = BroadcastEditorForm
        return UpdateView.get_form_class(self)

    def form_valid(self, form):

        changes = build_broadcast_changes(self.object, BROADCAST_TRACKFIELDS)
        self.object = form.save()

        if changes:
            action.send(self.request.user, verb=verbs.CHANGED_BROADCAST,
                        action_object=self.object,
                        changes=changes)

        messages.add_message(self.request, messages.SUCCESS,
                             "Sendung %(b)s wurde aktualisiert" % {'b': self.object})

        return UpdateView.form_valid(self, form)

    def get_success_url(self):
        return self.request.POST.get('referer_url', self.object.get_absolute_url())


@method_decorator(staff_member_required, name='dispatch')
class BroadcastDelete(SuccessMessageMixin, DeleteView):
    model = Broadcast
    template_name = 'program/broadcast/broadcast_confirm_delete.html'
    success_url = reverse_lazy('list-broadcasts')

    def get_context_data(self, **kwargs):
        context = super(BroadcastDelete, self).get_context_data(**kwargs)

        referer_view = get_referer_view(self.request, None, True)
        if referer_view:
            context['referer_url'] = referer_view
        return context

    def delete(self, request, *args, **kwargs):
        obj = self.get_object()

        action.send(self.request.user, verb=verbs.DELETED_BROADCAST,
                    action_object=obj, changes={
                        'program_id': "{} ({})".format(obj.program.pk, obj.program.name),
                        'start_time': obj.start_time.strftime("%Y-%m-%d %H:%M:%S"),
                        'end_time': obj.end_time.strftime("%Y-%m-%d %H:%M:%S")})

        messages.add_message(request, messages.SUCCESS,
                             "Sendung %(b)s wurde gelöscht" % {'b': obj})
        return super(BroadcastDelete, self).delete(request, *args, **kwargs)


@staff_member_required
def deleteBroadcastsByDates(request):
    context = {}

    form = BroadcastDeleteRangeForm(request.GET)
    context['form'] = form

    if request.method == 'POST':
        form_confirm = BroadcastDeleteRangeForm(request.POST, prefix="confirm")

        if form_confirm.is_valid():
            data = form_confirm.cleaned_data
            from_date = pytz.timezone(settings.TIME_ZONE) \
                .localize(datetime.combine(data.get('from_date'), time.min))
            to_date = pytz.timezone(settings.TIME_ZONE) \
                .localize(datetime.combine(data.get('to_date'), time.max))

            broadcasts = Broadcast.objects.filter(start_time__gte=from_date) \
                .filter(end_time__lte=to_date)

            for bc in broadcasts:
                bc.delete()

            messages.info(request, "{} Sendungen für Zeitraum {} {} wurden gelöscht."
                          .format(len(broadcasts), from_date.date(), to_date.date()))

    # GET
    else:
        if len(request.GET.keys()) > 0:
            if form.is_valid():
                form_confirm = BroadcastDeleteRangeForm(initial=form.data, prefix="confirm")
                form_confirm.fields['from_date'].widget = HiddenInput()
                form_confirm.fields['to_date'].widget = HiddenInput()
                context['form_confirm'] = form_confirm
            else:
                messages.warning(request, form.errors.as_text())

    return render(request, 'program/broadcast/broadcast_delete_daterange_form.html', context)


# Broadcast repeat (Wiederholungssendung)

@method_decorator(staff_member_required, name='dispatch')
class BroadcastRepeatCreate(CreateView):
    model = Broadcast
    form_class = BroadcastRepeatForm
    template_name = 'program/broadcast/broadcast_repeat_form.html'

    def get_context_data(self, **kwargs):
        context = super(BroadcastRepeatCreate, self).get_context_data(**kwargs)
        context['bcorigin'] = get_object_or_404(Broadcast, pk=self.kwargs.get('pk'))
        return context

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        firstrun = get_object_or_404(Broadcast, pk=self.kwargs.get('pk'))
        kwargs['instance'] = Broadcast(firstrun=firstrun)
        return kwargs

    def form_valid(self, form):

        self.object = form.save(commit=False)

        firstrun = self.object.firstrun
        self.object.program = firstrun.program
        self.object.end_time = self.object.start_time + (firstrun.end_time - firstrun.start_time)

        changes = build_broadcast_changes(self.object, BROADCAST_REPEAT_TRACKFIELDS)
        self.object = form.save()

        if changes:
            action.send(self.request.user, verb=verbs.CREATED_BROADCAST_REPEAT,
                        action_object=self.object,
                        target=self.object.firstrun,
                        changes=changes)

        return super(BroadcastRepeatCreate, self).form_valid(form)


@method_decorator(staff_member_required, name='dispatch')
class BroadcastRepeatUpdate(UpdateView):
    model = Broadcast
    form_class = BroadcastRepeatForm
    template_name = 'program/broadcast/broadcast_repeat_form.html'

    def get_context_data(self, **kwargs):
        context = super(BroadcastRepeatUpdate, self).get_context_data(**kwargs)
        referer_view = get_referer_view(self.request, None, True)
        if referer_view:
            context['referer_url'] = referer_view
        return context

    def form_valid(self, form):
        self.object = form.save(commit=False)
        self.object.end_time = self.object.start_time + \
            (self.object.firstrun.end_time - self.object.firstrun.start_time)

        changes = build_broadcast_changes(self.object, BROADCAST_REPEAT_TRACKFIELDS)
        self.object = form.save()

        if changes:
            action.send(self.request.user, verb=verbs.CHANGED_BROADCAST_REPEAT,
                        action_object=self.object,
                        target=self.object.firstrun,
                        changes=changes)

        messages.add_message(self.request, messages.SUCCESS,
                             "Wiederholungssendung %(b)s wurde aktualisiert" % {'b': self.object})
        return super(BroadcastRepeatUpdate, self).form_valid(form)

    def get_success_url(self):
        return self.request.POST.get('referer_url', self.object.get_absolute_url())


@method_decorator(staff_member_required, name='dispatch')
class BroadcastRepeatDelete(SuccessMessageMixin, DeleteView):
    model = Broadcast
    template_name = 'program/broadcast/broadcast_repeat_confirm_delete.html'

    def get_context_data(self, **kwargs):
        context = super(BroadcastRepeatDelete, self).get_context_data(**kwargs)
        referer_view = get_referer_view(self.request, None, True)
        if referer_view:
            context['referer_url'] = referer_view
        return context

    def delete(self, request, *args, **kwargs):
        obj = self.get_object()

        action.send(self.request.user, verb=verbs.DELETED_BROADCAST_REPEAT,
                    action_object=obj,
                    target=obj.firstrun,
                    changes={'start_time': obj.start_time.strftime("%Y-%m-%d %H:%M:%S"),
                             'end_time': obj.end_time.strftime("%Y-%m-%d %H:%M:%S")})

        messages.add_message(self.request, messages.SUCCESS,
                             "Wiederholungssendung %(b)s wurde gelöscht" % {'b': obj})
        return super(BroadcastRepeatDelete, self).delete(request, *args, **kwargs)

    def get_success_url(self):
        return self.request.POST.get('referer_url', self.object.firstrun.get_absolute_url())


def build_broadcast_changes(broadcast: Broadcast, trackfields):

    changes = {}

    for trackfield in trackfields:

        if trackfield in broadcast.tracker.changed().keys():

            changes[trackfield] = getattr(broadcast, trackfield)

            if not changes[trackfield] and broadcast.tracker.previous(trackfield) is None:
                del changes[trackfield]
                continue

            if trackfield in ['start_time', 'end_time']:
                changes[trackfield] = changes[trackfield].strftime("%Y-%m-%d %H:%M:%S")

            if trackfield == 'program_id':
                program_new = RadioProgram.objects.get(pk=changes[trackfield])

                if(program_new):
                    changes[trackfield] = "{} ({})".format(program_new.pk, program_new.name)

    return changes
