from django.views.generic.edit import UpdateView, DeleteView, CreateView
from program.models import BroadcastRepeatTemplate, BroadcastTemplate,\
    BroadcastTemplateRegularItem, BroadcastItem
from django.contrib import messages
from django.contrib.messages.views import SuccessMessageMixin
from django.shortcuts import get_object_or_404, redirect
from django_filters.views import FilterView
from program.filters import BroadcastTemplateFilter
from django.views.generic.detail import DetailView
from program.forms import BroadcastTemplateForm,\
    BroadcastTemplateRegularItemForm
from django.urls.base import reverse
from django.utils.decorators import method_decorator
from django.contrib.admin.views.decorators import staff_member_required


@method_decorator(staff_member_required, name='dispatch')
class BroadcastTemplateList(FilterView):
    filterset_class = BroadcastTemplateFilter
    context_object_name = 'broadcasttemplates'
    paginate_by = 10
    template_name = 'program/broadcasttemplate/broadcasttemplate_list.html'
    strict = False


@method_decorator(staff_member_required, name='dispatch')
class BroadcastTemplateDetail(DetailView):
    model = BroadcastTemplate
    template_name = 'program/broadcasttemplate/broadcasttemplate_detail.html'


@method_decorator(staff_member_required, name='dispatch')
class BroadcastTemplateCreate(CreateView):
    model = BroadcastTemplate
    form_class = BroadcastTemplateForm
    template_name = 'program/broadcasttemplate/broadcasttemplate_form.html'


@method_decorator(staff_member_required, name='dispatch')
class BroadcastTemplateUpdate(UpdateView):
    model = BroadcastTemplate
    form_class = BroadcastTemplateForm
    template_name = 'program/broadcasttemplate/broadcasttemplate_form.html'


@method_decorator(staff_member_required, name='dispatch')
class BroadcastTemplateDelete(DeleteView):
    model = BroadcastTemplate
    template_name = 'program/broadcasttemplate/broadcasttemplate_confirm_delete.html'

    def get_success_url(self):
        return reverse('list-broadcast-templates')


@method_decorator(staff_member_required, name='dispatch')
class BroadcastTemplateRegularItemCreate(CreateView):
    model = BroadcastTemplateRegularItem
    form_class = BroadcastTemplateRegularItemForm
    template_name = 'program/broadcasttemplate/broadcasttemplateregularitem_form.html'

    def get_context_data(self, **kwargs):
        context = super(BroadcastTemplateRegularItemCreate, self).get_context_data(**kwargs)
        context['bctmpl'] = get_object_or_404(BroadcastTemplate, pk=self.kwargs.get('bctmpl_pk'))
        return context

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        bctmpl = get_object_or_404(BroadcastTemplate, pk=self.kwargs.get('bctmpl_pk'))
        kwargs['instance'] = BroadcastTemplateRegularItem(broadcasttemplate=bctmpl)
        return kwargs


@method_decorator(staff_member_required, name='dispatch')
class BroadcastTemplateRegularItemUpdate(UpdateView):
    model = BroadcastTemplateRegularItem
    form_class = BroadcastTemplateRegularItemForm
    template_name = 'program/broadcasttemplate/broadcasttemplateregularitem_form.html'

    def get_context_data(self, **kwargs):
        context = super(BroadcastTemplateRegularItemUpdate, self).get_context_data(**kwargs)
        context['bctmpl'] = self.object.broadcasttemplate
        return context


@method_decorator(staff_member_required, name='dispatch')
class BroadcastTemplateRegularItemDelete(SuccessMessageMixin, DeleteView):
    model = BroadcastTemplateRegularItem
    template_name = 'program/broadcasttemplate/broadcasttemplateregularitem_confirm_delete.html'

    def delete(self, request, *args, **kwargs):
        obj = self.get_object()
        messages.success(self.request, self.success_message % obj.__dict__)
        return super(BroadcastTemplateRegularItemDelete, self).delete(request, *args, **kwargs)

    def get_success_url(self):
        return self.object.broadcasttemplate.get_absolute_url()


@staff_member_required
def move_regular_item_of_broadcast_template(request, pk, action):

    def top():
        items = BroadcastTemplateRegularItem.objects.\
            filter(broadcasttemplate=item.broadcasttemplate).order_by('position')
        if item.position == 1:
            messages.add_message(request, messages.WARNING,
                                 "Der gewählte Eintrag ist bereits am Anfang der Liste")
            return

        for i in items:
            if i.position < item.position:
                i.position += 1
                i.save()
        item.position = 1
        item.save()
        return

    def up():
        if item.position == 1:
            messages.add_message(request, messages.WARNING,
                                 "Der gewählte Eintrag ist bereits am Anfang der Liste")
            return

        upper_item = BroadcastTemplateRegularItem.objects\
            .filter(broadcasttemplate=item.broadcasttemplate)\
            .filter(position=item.position-1).get()
        upper_item.position += 1
        upper_item.save()
        item.position -= 1
        item.save()
        return

    def down():
        try:
            lower_item = BroadcastTemplateRegularItem.objects\
                .filter(broadcasttemplate=item.broadcasttemplate)\
                .filter(position=item.position+1).get()
        except BroadcastItem.DoesNotExist:
            messages.add_message(request, messages.WARNING,
                                 "Der gewählte Eintrag ist bereits am Ende der Liste")
            return

        lower_item.position -= 1
        lower_item.save()
        item.position += 1
        item.save()
        return

    def bottom():
        items = BroadcastTemplateRegularItem.objects\
            .filter(broadcasttemplate=item.broadcasttemplate).order_by('position')

        if item.position == len(items):
            messages.add_message(request, messages.WARNING,
                                 "Der gewählte Eintrag ist bereits am Ende der Sequenz")

        for i in items:
            if i.position > item.position:
                i.position -= 1
                i.save()
        item.position = len(items)
        item.save()
        return

    switcher = {
        'top': top,
        'up': up,
        'down': down,
        'bottom':  bottom
    }

    item = get_object_or_404(BroadcastTemplateRegularItem, pk=pk)

    if action not in switcher.keys():
        messages.add_message(request, messages.WARNING,
                             "Der Wert {0} ist nicht gültig für den Parameter move").format(action)
    else:
        func = switcher.get(action, "nothing")
        func()
    return redirect(
        reverse(
            'show-broadcast-template',
            kwargs={'pk': item.broadcasttemplate.pk}) + '#regularitems'
        )


@method_decorator(staff_member_required, name='dispatch')
class BroadcastRepeatTemplateCreate(CreateView):
    model = BroadcastRepeatTemplate
    template_name = 'program/broadcasttemplate/broadcastrepeattemplate_form.html'
    fields = ['plusweek', 'weekday', 'start_time']

    def get_form_kwargs(self):
        kwargs = super().get_form_kwargs()
        origin = get_object_or_404(BroadcastTemplate, pk=self.kwargs.get('bctmpl_pk'))
        kwargs['instance'] = BroadcastRepeatTemplate(origin=origin)
        return kwargs


@method_decorator(staff_member_required, name='dispatch')
class BroadcastRepeatTemplateUpdate(UpdateView):
    model = BroadcastRepeatTemplate
    template_name = 'program/broadcasttemplate/broadcastrepeattemplate_form.html'
    fields = ['plusweek', 'weekday', 'start_time']


@method_decorator(staff_member_required, name='dispatch')
class BroadcastRepeatTemplateDelete(SuccessMessageMixin, DeleteView):
    template_name = 'program/broadcasttemplate/broadcastrepeattemplate_confirm_delete.html'
    model = BroadcastRepeatTemplate

    def delete(self, request, *args, **kwargs):
        obj = self.get_object()
        messages.success(self.request, self.success_message % obj.__dict__)
        return super(BroadcastRepeatTemplateDelete, self).delete(request, *args, **kwargs)

    def get_success_url(self):
        return self.object.origin.get_absolute_url()
