from django.db.models import Exists, OuterRef, Q
from django.utils.timezone import now
from django_filters import rest_framework as filters
from django_filters import CharFilter
from taggit.forms import TagField

from audiothek.models import Audiofile, RadioReport
from program.models import Broadcast, RadioProgram
from core.models import ContentTag


class NumberInFilter(filters.BaseInFilter, filters.NumberFilter):
    pass


class HasTagFilter(CharFilter):
    field_class = TagField

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('lookup_expr', 'in')
        super().__init__(*args, field_name='tags__slug', distinct=True, **kwargs)


class ContentTagFilter(filters.FilterSet):
    class Meta:
        model = ContentTag
        fields = tuple()


class BroadcastFilter(filters.FilterSet):
    class Meta:
        model = Broadcast
        fields = {
            'start_time': ('lt', 'gt',),
            'end_time': ('lt', 'gt',),
        }
        extra_fields = {
            'series': ('exact', ),
        }

    series = filters.ModelChoiceFilter(field_name='program', queryset=RadioProgram.objects.all())


class ReportFilter(filters.FilterSet):
    class Meta:
        model = RadioReport
        fields = {
                'publication_time': ('gt',),
                'unpublication_time': ('lt',),
        }
        extra_fields = {
                'broadcast': ('exact',),
                'series': ('exact',),
                'is_available': ('exact',),
        }

    broadcast = filters.ModelChoiceFilter(
            queryset=Broadcast.objects.all(), method='filter_broadcast', label='Sendung')
    series = filters.ModelChoiceFilter(
            queryset=RadioProgram.objects.all(), method='filter_series', label='Sendereihe')
    is_available = filters.BooleanFilter(method='filter_available', label='Verfügbar')
    tags = HasTagFilter()
    omit = NumberInFilter(field_name='id', lookup_expr='in', exclude=True,
                          label='Einzelne IDs ausschließen')
    id = NumberInFilter(field_name='id', lookup_expr='in', label='Auf IDs begrenzen')
    keywords = filters.CharFilter(method='filter_keywords', label='Stichworte')

    def filter_broadcast(self, queryset, name, value):
        return queryset.filter(broadcasts=value)

    def filter_series(self, queryset, name, value):
        return queryset.filter(broadcasts__program=value)

    def filter_available(self, queryset, name, value):
        query = Q(publication_time__isnull=False) & Q(publication_time__lt=now()) & (
            Q(unpublication_time__isnull=True) | Q(unpublication_time__gt=now()))
        queryset = queryset.annotate(
            has_audiofiles=Exists(Audiofile.objects.filter(report=OuterRef('pk'))))
        if value:
            return queryset.filter(query, has_audiofiles=True)
        else:
            return queryset.exclude(query, has_audiofiles=True)

    def filter_keywords(self, queryset, name, value):
        for keyword in value.split():
            queryset = queryset.filter(
                    Q(title__icontains=keyword) | Q(description__icontains=keyword))
        return queryset
