from taggit.models import Tag
from rest_framework import serializers
from rest_framework.reverse import reverse

from audiothek.models import Audiofile, RadioReport
from core.models import License, ContentTag
from program.models import Broadcast, RadioProgram, BroadcastItem


class AudiofileSourceSerializer(serializers.ModelSerializer):
    src = serializers.SerializerMethodField()
    bitrate = serializers.SerializerMethodField()
    container = serializers.SerializerMethodField()
    codec = serializers.SerializerMethodField()

    def get_container(self, obj):
        return obj.container or None

    def get_codec(self, obj: Audiofile):
        return obj.codec

    def get_bitrate(self, obj: Audiofile):
        return obj.bitrate / 1000

    def get_src(self, obj: Audiofile):
        return reverse('stream-audiofile', args=(obj.pk,), request=self.context['request'])

    class Meta:
        model = Audiofile
        fields = ('bitrate', 'codec', 'container', 'src')


class AudiofileSerializer(serializers.ModelSerializer):
    sources = AudiofileSourceSerializer(source='*', many=True)
    duration = serializers.IntegerField(source='length')

    class Meta:
        model = Audiofile
        fields = ('id', 'description', 'duration', 'sources')


class BroadcastSerializer(serializers.ModelSerializer):
    series_id = serializers.PrimaryKeyRelatedField(source='program', read_only=True)
    recording_ids = serializers.PrimaryKeyRelatedField(source='reports', many=True, read_only=True)

    class Meta:
        model = Broadcast
        fields = (
                'id', 'series_id', 'subtitle', 'description', 'start_time', 'end_time',
                'recording_ids', 'cover_image'
        )


class BroadcastItemSerializer(serializers.ModelSerializer):
    broadcast_id = serializers.PrimaryKeyRelatedField(source='broadcast', read_only=True)
    title = serializers.SerializerMethodField()
    report_id = serializers.PrimaryKeyRelatedField(source='report', read_only=True)
    report_type = serializers.SerializerMethodField()

    def get_title(self, obj):
        if obj.report:
            return obj.report.title
        elif obj.regular:
            return obj.regular.title
        elif obj.moderation:
            return obj.moderation.title

    def get_report_type(self, obj):

        if obj.report:
            return obj.report.type
        return ""

    class Meta:
        model = BroadcastItem
        fields = (
                'broadcast_id', 'position', 'title', 'time', 'report_id', 'report_type'
        )


class LicenseSerializer(serializers.ModelSerializer):
    class Meta:
        model = License
        fields = ('name', 'identifier', 'url')


class ProgramSerializer(serializers.ModelSerializer):
    class Meta:
        model = RadioProgram
        fields = ('id', 'name', 'description', 'teaser', 'cover_image',)


class MinimalTagSerializer(serializers.ModelSerializer):
    class Meta:
        model = ContentTag
        fields = ('name', 'slug')


class ContentTagSerializer(MinimalTagSerializer):
    weight = serializers.SerializerMethodField()

    def get_weight(self, obj):
        # TODO: calculate weight based on how often this tag
        #       was used on any model within $time_frame and
        #       a float between 0 and 1
        #       (0 == not used, 1 == only used tag)
        return 0

    class Meta:
        model = Tag
        fields = ('name', 'slug', 'weight')


class ReportSerializer(serializers.ModelSerializer):
    audio = AudiofileSerializer(source='audiofiles', many=True)
    license = LicenseSerializer(required=False)
    tags = MinimalTagSerializer(many=True)
    publication = serializers.SerializerMethodField()
    broadcast_ids = serializers.PrimaryKeyRelatedField(source='broadcasts',
                                                       read_only=True, many=True)

    def get_publication(self, obj):
        return {
            'start': obj.publication_time,
            'end': obj.unpublication_time
        }

    class Meta:
        model = RadioReport
        fields = (
            'audio',
            'broadcast_ids',
            'can_be_streamed',
            'cover_image',
            'description',
            'id',
            'license',
            'publication',
            'production_date',
            'tags',
            'title',
            'type',
        )
