from django.http import HttpResponseGone
from django.shortcuts import get_object_or_404
from django.views.generic import View
from imagekit import ImageSpec
from imagekit.cachefiles import ImageCacheFile
from imagekit.processors import ResizeToFill
from rest_framework import viewsets, generics

from audiothek.models import Audiofile, RadioReport
from core.http import HttpResponseSendfile
from core.models import Image, ContentTag
from program.models import Broadcast, RadioProgram, BroadcastItem
from rest_api.frontend.filters import BroadcastFilter, ReportFilter, ContentTagFilter
from rest_api.frontend.serializers import BroadcastSerializer, ProgramSerializer, \
        ReportSerializer, BroadcastItemSerializer, ContentTagSerializer
from rest_framework.exceptions import ParseError
from program.querysets import query_broadcasts_of_day
from program.broadcast_transform import adjust_gapfill_broadcasts
from datetime import datetime
from django.core.exceptions import ObjectDoesNotExist

SUPPORTED_IMAGE_RENDER_FORMATS = ('PNG', 'JPEG', 'WEBP')


class BroadcastViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = Broadcast.objects.all().order_by('start_time')
    serializer_class = BroadcastSerializer
    filterset_class = BroadcastFilter


class BroadcastDayViewSet(generics.ListAPIView):
    serializer_class = BroadcastSerializer
    pagination_class = None

    def get_queryset(self):
        try:
            day = datetime.strptime(self.kwargs.get("date"), "%Y-%m-%d")
        except ValueError:
            raise ParseError("invalid date", 400)

        qs = Broadcast.objects.all().order_by('start_time')
        qs = adjust_gapfill_broadcasts(query_broadcasts_of_day(qs, day))
        return qs


class BroadcastItemsViewSet(generics.ListAPIView):
    serializer_class = BroadcastItemSerializer

    def get_queryset(self):

        try:
            bcid = int(self.kwargs.get("pk"))
            bc = Broadcast.objects.get(pk=bcid)
        except ValueError:
            raise ParseError("non numeric id", 400)
        except ObjectDoesNotExist:
            raise ParseError("no broadcast found by id {}".format(bcid), 400)

        return BroadcastItem.objects.filter(broadcast=bc).filter(public=True).order_by('position')


class ProgramViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = RadioProgram.objects.all()
    serializer_class = ProgramSerializer


class RenderImageView(View):
    def get_image(self):
        return get_object_or_404(Image, pk=self.kwargs.get('pk'))

    def get(self, *args, **kwargs):
        image_file = self.get_image().image_file
        params = self._get_params()
        if params['file_format'] or params['width'] and params['height']:
            image_file = self._process_image(image_file, **params)
        return HttpResponseSendfile(image_file)

    def _get_params(self):
        query_params = self.request.GET

        try:
            width = int(query_params.get('width', ''))
            height = int(query_params.get('height', ''))
            if width <= 0 or height <= 0:
                raise ValueError('width and height must be positive')
        except ValueError:
            width = height = None

        file_format = query_params.get('format')
        if file_format and file_format.upper() not in SUPPORTED_IMAGE_RENDER_FORMATS:
            file_format = None

        strategy = query_params.get('strategy')
        return {'width': width, 'height': height, 'file_format': file_format,
                'strategy': strategy}

    def _process_image(self, image_file, width, height, file_format, strategy):
        spec = ImageSpec(image_file)
        if file_format:
            spec.format = file_format
        if width and height:
            spec.processors = [ResizeToFill(width, height)]
        cache_file = ImageCacheFile(spec)
        cache_file.generate()
        return cache_file


class ReportViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = RadioReport.objects.all().order_by('-publication_time', '-production_date')
    serializer_class = ReportSerializer
    filterset_class = ReportFilter


class TagViewSet(viewsets.ReadOnlyModelViewSet):
    queryset = ContentTag.objects.all()
    serializer_class = ContentTagSerializer
    filterset_class = ContentTagFilter


class StreamAudiofileView(View):
    def get_audiofile(self):
        return get_object_or_404(Audiofile, pk=self.kwargs.get('pk'))

    def get(self, *args, **kwargs):
        audiofile = self.get_audiofile()
        report = audiofile.report or audiofile.transcoded_from.report

        if report.can_be_streamed:
            return HttpResponseSendfile(audiofile.audiofile,
                                        filename=self.request.GET.get('download', None))
        else:
            return HttpResponseGone()
